#!/usr/bin/python
# Copyright (C) 2025 Ford Motor Company
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for basic Source/Replica communication'''

import os
import sys
import textwrap
import enum
import gc

import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUrl, qWarning
from PySide6.QtRemoteObjects import (QRemoteObjectHost, QRemoteObjectNode, QRemoteObjectReplica,
                                     QRemoteObjectPendingCall, RepFile, getCapsuleCount)
from PySide6.QtTest import QSignalSpy, QTest

from test_shared import wrap_tests_for_cleanup
from helper.usesqapplication import UsesQApplication

contents = """
class Simple
{
    PROP(int i = 2);
    PROP(float f = -1. READWRITE);
    SIGNAL(random(int i));
    SLOT(void reset());
    SLOT(int add(int i));
};
"""


class QBasicTest(UsesQApplication):
    '''Test case for basic source/replica communication'''
    def setUp(self):
        # Separate output to make debugging easier
        qWarning(f"\nSet up {self.__class__.__qualname__}")
        super().setUp()
        '''Set up test environment'''
        if hasattr(self.__class__, "contents"):
            qWarning(f"Using class contents >{self.__class__.contents}<")
            self.rep = RepFile(self.__class__.contents)
        else:
            self.rep = RepFile(contents)
        self.host = QRemoteObjectHost(QUrl("tcp://127.0.0.1:0"))
        self.host.setObjectName("host")
        self.node = QRemoteObjectNode()
        self.node.setObjectName("node")
        self.node.connectToNode(self.host.hostUrl())     # pick up the url with the assigned port

    def compare_properties(self, instance, values):
        '''Compare properties of instance with values'''
        self.assertEqual(instance.i, values[0])
        self.assertAlmostEqual(instance.f, values[1], places=5)

    def default_setup(self):
        '''Set up default test environment'''
        replica = self.node.acquire(self.rep.replica["Simple"])
        # Make sure the replica is initialized with default values
        self.compare_properties(replica, [2, -1])
        self.assertEqual(replica.isInitialized(), False)
        source = self.rep.source["Simple"]()
        # Make sure the source is initialized with default values
        self.compare_properties(source, [2, -1])
        return replica, source

    def tearDown(self):
        self.assertEqual(getCapsuleCount(), 0)
        self.app.processEvents()
        super().tearDown()
        # Separate output to make debugging easier
        qWarning(f"Tore down {self.__class__.__qualname__}\n")


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class ReplicaInitialization(QBasicTest):
    def test_ReplicaInitialization(self):
        replica, source = self.default_setup()
        source.i = -1
        source.f = 3.14
        self.compare_properties(source, [-1, 3.14])
        init_spy = QSignalSpy(replica.initialized)
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        self.assertEqual(replica.state(), QRemoteObjectReplica.State.Valid)
        # Make sure the replica values are updated to the source values
        self.compare_properties(replica, [-1, 3.14])
        self.assertEqual(init_spy.count(), 1)
        self.assertEqual(replica.isInitialized(), True)


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class SourcePropertyChange(QBasicTest):
    def test_SourcePropertyChange(self):
        replica, source = self.default_setup()
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        # Make sure the replica values are unchanged since the source had the same values
        self.compare_properties(replica, [2, -1])
        source_spy = QSignalSpy(source.iChanged)
        replica_spy = QSignalSpy(replica.iChanged)
        source.i = 42
        self.assertEqual(source_spy.count(), 1)
        # Make sure the source value is updated
        self.compare_properties(source, [42, source.f])
        self.assertTrue(replica_spy.wait(1000))
        self.assertEqual(replica_spy.count(), 1)
        # Make sure the replica value is updated
        self.compare_properties(replica, [42, replica.f])


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class ReplicaPropertyChange(QBasicTest):
    def test_ReplicaPropertyChange(self):
        replica, source = self.default_setup()
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        # Make sure push methods are working
        source_spy = QSignalSpy(source.iChanged)
        replica_spy = QSignalSpy(replica.iChanged)
        replica.pushI(11)
        # # Let eventloop run to update the source and verify the values
        self.assertTrue(source_spy.wait(1000))
        self.assertEqual(source_spy.count(), 1)
        self.compare_properties(source, [11, source.f])
        # Let eventloop run to update the replica and verify the values
        self.assertTrue(replica_spy.wait(1000))
        self.assertEqual(replica_spy.count(), 1)
        self.compare_properties(replica, [11, replica.f])

        # Test setter on replica
        source_spy = QSignalSpy(source.fChanged)
        replica_spy = QSignalSpy(replica.fChanged)
        replica.f = 4.2
        # Make sure the replica values are ** NOT CHANGED ** since the eventloop hasn't run
        self.compare_properties(replica, [11, -1])
        # Let eventloop run to update the source and verify the values
        self.assertTrue(source_spy.wait(1000))
        self.assertEqual(source_spy.count(), 1)
        self.compare_properties(source, [source.i, 4.2])
        # Let eventloop run to update the replica and verify the values
        self.assertTrue(replica_spy.wait(1000))
        self.assertEqual(replica_spy.count(), 1)
        self.compare_properties(replica, [replica.i, 4.2])


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class DerivedReplicaPropertyChange(QBasicTest):
    def test_DerivedReplicaPropertyChange(self):
        # Don't use default_setup(), instead create a derived replica
        Replica = self.rep.replica["Simple"]

        class DerivedReplica(Replica):
            pass

        replica = self.node.acquire(DerivedReplica)
        # Make sure the replica is initialized with default values
        self.compare_properties(replica, [2, -1])
        self.assertEqual(replica.isInitialized(), False)
        source = self.rep.source["Simple"]()
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class ReplicaSlotNotImplementedChange(QBasicTest):
    def test_ReplicaSlotNotImplementedChange(self):
        replica, source = self.default_setup()
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        # Ideally this would fail as the slot is not implemented on the source
        res = replica.reset()
        self.assertEqual(type(res), type(None))
        QTest.qWait(100)  # Wait for 100 ms for async i/o.  There isn't a signal to wait on
        res = replica.add(5)
        self.assertEqual(type(res), QRemoteObjectPendingCall)


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class ReplicaSlotImplementedChange(QBasicTest):
    def test_ReplicaSlotImplementedChange(self):
        replica = self.node.acquire(self.rep.replica["Simple"])
        replica.setObjectName("replica")

        class Source(self.rep.source["Simple"]):
            def __init__(self):
                super().__init__()
                self.i = 6
                self.f = 3.14

            def reset(self):
                self.i = 0
                self.f = 0

            def add(self, i):
                return self.i + i

        source = Source()
        source.setObjectName("source")
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        self.compare_properties(source, [6, 3.14])
        self.compare_properties(replica, [6, 3.14])
        replica_spy = QSignalSpy(replica.iChanged)
        res = replica.reset()
        self.assertEqual(type(res), type(None))
        self.assertEqual(replica_spy.wait(1000), True)
        self.compare_properties(source, [0, 0])
        self.compare_properties(replica, [0, 0])
        res = replica.add(5)
        self.assertEqual(type(res), QRemoteObjectPendingCall)
        self.assertEqual(res.waitForFinished(1000), True)
        self.assertEqual(res.returnValue(), 5)


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class RefCountTest(QBasicTest):
    contents = textwrap.dedent("""\
        POD MyPOD{
            ENUM class Position : unsigned short {position1=1, position2=2, position3=4}
            Position pos,
            QString name
        }
        class Simple
        {
            ENUM Position {Left, Right, Top, Bottom}
            PROP(MyPOD myPod);
            PROP(Position pos);
        }
        """)

    def test_RefCount(self):
        # Once the rep file is loaded, we should be tracking 4 converter capsules
        # - 1 for the POD itself
        # - 1 for the enum in the POD
        # - 1 for the enum in the Source
        # - 1 for the enum in the Replica
        # We should be tracking 3 qobject capsules (POD, Replica, Source)
        # Note: Source and Replica are distinct types, so Source::EPosition and
        # Replica::EPosition are distinct as well.
        # Note 2: The name of the enum ("Position") can be reused for different
        # types in different classes as shown above.
        self.assertEqual(getCapsuleCount(), 7)
        MyPod = self.rep.pod["MyPOD"]
        self.assertTrue(isinstance(MyPod, type))
        self.assertTrue(issubclass(MyPod, tuple))
        MyEnum = MyPod.get_enum("Position")
        self.assertTrue(isinstance(MyEnum, type))
        self.assertTrue(issubclass(MyEnum, enum.Enum))
        e = MyEnum(4)  # noqa: F841
        Source = self.rep.source["Simple"]
        source = Source()  # noqa: F841
        source = None  # noqa: F841
        Source = None
        Replica = self.rep.replica["Simple"]
        replica = self.node.acquire(Replica)  # noqa: F841
        replica = None  # noqa: F841
        Replica = None
        MyEnum = None
        MyPod = None
        self.rep = None
        e = None  # noqa: F841
        gc.collect()
        # The enum and POD capsules will only be deleted (garbage collected) if
        # the types storing them (RepFile, Replica and Source) are garbage
        # collected first.
        self.assertEqual(getCapsuleCount(), 0)


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class EnumTest(QBasicTest):
    contents = textwrap.dedent("""\
        POD MyPOD{
            ENUM class Position : unsigned short {position1=1, position2=2, position3=4}
            Position pos,
            QString name
        }
        class Simple
        {
            ENUM Position {Left, Right, Top, Bottom}
            PROP(MyPOD myPod);
            PROP(Position pos);
        }
        """)

    def test_Enum(self):
        MyPod = self.rep.pod["MyPOD"]
        self.assertTrue(isinstance(MyPod, type))
        self.assertTrue(issubclass(MyPod, tuple))
        PodEnum = MyPod.get_enum("Position")
        self.assertTrue(isinstance(PodEnum, type))
        self.assertTrue(issubclass(PodEnum, enum.Enum))
        t = (PodEnum(4), "test")
        myPod = MyPod(*t)
        with self.assertRaises(ValueError):
            myPod = MyPod(PodEnum(0), "thing")  # 0 isn't a valid enum value
        myPod = MyPod(PodEnum(2), "thing")
        self.assertEqual(myPod.pos, PodEnum.position2)
        replica = self.node.acquire(self.rep.replica["Simple"])
        replica.setObjectName("replica")
        source = self.rep.source["Simple"]()
        source.setObjectName("source")
        source.myPod = (PodEnum.position2, "Hello")
        SourceEnum = source.get_enum("Position")
        self.assertTrue(isinstance(SourceEnum, type))
        self.assertTrue(issubclass(SourceEnum, enum.Enum))
        source.pos = SourceEnum.Top
        self.assertEqual(source.myPod, (PodEnum.position2, "Hello"))
        self.assertNotEqual(source.pos, 2)
        self.host.enableRemoting(source)
        self.assertEqual(replica.waitForSource(1000), True)
        self.assertEqual(replica.myPod, (PodEnum.position2, "Hello"))
        ReplicaEnum = replica.get_enum("Position")
        # Test invalid comparisons
        self.assertNotEqual(replica.pos, 2)
        self.assertNotEqual(replica.pos, SourceEnum.Top)
        self.assertNotEqual(replica.myPod, (SourceEnum(2), "Hello"))
        self.assertNotEqual(replica.myPod, (ReplicaEnum(2), "Hello"))
        self.assertNotEqual(replica.myPod, (2, "Hello"))
        # Test valid comparisons to Replica enum
        self.assertEqual(replica.pos, ReplicaEnum.Top)
        self.assertEqual(replica.myPod, (PodEnum(2), "Hello"))


@wrap_tests_for_cleanup(extra=['rep', 'host', 'node'])
class PodTest(QBasicTest):
    contents = textwrap.dedent("""\
        POD MyPod(int i, QString s)

        class Simple
        {
            PROP(MyPod pod);
        }
        """)

    def test_Pod(self):
        MyPod = self.rep.pod["MyPod"]
        self.assertTrue(isinstance(MyPod, type))
        self.assertTrue(issubclass(MyPod, tuple))
        source = self.rep.source["Simple"]()
        t = (42, "Hello")
        pod = MyPod(*t)
        source.pod = t
        self.assertEqual(source.pod, t)
        self.assertEqual(source.pod, pod)
        source.pod = pod
        self.assertEqual(source.pod, t)
        self.assertEqual(source.pod, pod)
        with self.assertRaises(ValueError):
            source.pod = (11, "World", "!")
        with self.assertRaises(TypeError):
            source.pod = MyPod("Hello", "World")
        self.assertEqual(source.pod, pod)
        self.assertTrue(isinstance(pod, MyPod))
        self.assertEqual(pod.i, 42)
        self.assertEqual(pod.s, "Hello")
        self.assertTrue(isinstance(source.pod, MyPod))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2010 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import IntEnum
import sys

from PySide6.QtCore import (Property, QEasingCurve, QObject, QPropertyAnimation,
                            QPoint, QPointF, QRect, QRectF, QSize, Qt)
from PySide6.QtGui import (QBrush, QColor, QIcon, QLinearGradient, QPainter,
                           QPainterPath, QPixmap)
from PySide6.QtWidgets import (QApplication, QGraphicsPixmapItem,
                               QGraphicsItem, QGraphicsScene,
                               QListWidgetItem, QWidget)

from ui_form import Ui_Form


class PathType(IntEnum):
    LINEAR_PATH = 0
    CIRCLE_PATH = 1


def createEasingCurve(curveType):
    curve = QEasingCurve(curveType)
    if curveType == QEasingCurve.Type.BezierSpline:
        curve.addCubicBezierSegment(QPointF(0.4, 0.1), QPointF(0.6, 0.9), QPointF(1.0, 1.0))
    elif curveType == QEasingCurve.Type.TCBSpline:
        curve.addTCBSegment(QPointF(0.0, 0.0), 0, 0, 0)
        curve.addTCBSegment(QPointF(0.3, 0.4), 0.2, 1, -0.2)
        curve.addTCBSegment(QPointF(0.7, 0.6), -0.2, 1, 0.2)
        curve.addTCBSegment(QPointF(1.0, 1.0), 0, 0, 0)
    return curve


class Animation(QPropertyAnimation):
    def __init__(self, target, prop, parent=None):
        super().__init__(target, prop, parent)
        self.set_path_type(PathType.LINEAR_PATH)

    def set_path_type(self, pathType):
        self._pathType = pathType
        self._path = QPainterPath()

    def updateCurrentTime(self, currentTime):
        if self._pathType == PathType.CIRCLE_PATH:
            if self._path.isEmpty():
                end = self.endValue()
                start = self.startValue()
                self._path.moveTo(start)
                self._path.addEllipse(QRectF(start, end))

            dura = self.duration()
            if dura == 0:
                progress = 1.0
            else:
                progress = (((currentTime - 1) % dura) + 1) / float(dura)

            eased_progress = self.easingCurve().valueForProgress(progress)
            if eased_progress > 1.0:
                eased_progress -= 1.0
            elif eased_progress < 0:
                eased_progress += 1.0

            pt = self._path.pointAtPercent(eased_progress)
            self.updateCurrentValue(pt)
            self.valueChanged.emit(pt)
        else:
            super(Animation, self).updateCurrentTime(currentTime)


# PySide6 doesn't support deriving from more than one wrapped class so we use
# composition and delegate the property.
class Pixmap(QObject):
    def __init__(self, pix):
        super().__init__()

        self.pixmap_item = QGraphicsPixmapItem(pix)
        self.pixmap_item.setCacheMode(QGraphicsItem.CacheMode.DeviceCoordinateCache)

    def set_pos(self, pos):
        self.pixmap_item.setPos(pos)

    def get_pos(self):
        return self.pixmap_item.pos()

    pos = Property(QPointF, get_pos, set_pos)


class Window(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self._iconSize = QSize(64, 64)
        self._scene = QGraphicsScene()

        m_ui = Ui_Form()
        m_ui.setupUi(self)
        m_ui.easingCurvePicker.setIconSize(self._iconSize)
        m_ui.easingCurvePicker.setMinimumHeight(self._iconSize.height() + 50)
        m_ui.buttonGroup.setId(m_ui.lineRadio, 0)
        m_ui.buttonGroup.setId(m_ui.circleRadio, 1)

        dummy = QEasingCurve()
        m_ui.periodSpinBox.setValue(dummy.period())
        m_ui.amplitudeSpinBox.setValue(dummy.amplitude())
        m_ui.overshootSpinBox.setValue(dummy.overshoot())

        m_ui.easingCurvePicker.currentRowChanged.connect(self.curve_changed)
        m_ui.buttonGroup.idClicked.connect(self.path_changed)
        m_ui.periodSpinBox.valueChanged.connect(self.period_changed)
        m_ui.amplitudeSpinBox.valueChanged.connect(self.amplitude_changed)
        m_ui.overshootSpinBox.valueChanged.connect(self.overshoot_changed)

        self._ui = m_ui
        self.create_curve_icons()

        pix = QPixmap(':/qt-project.org/logos/pysidelogo.png')
        self._item = Pixmap(pix)
        self._scene.addItem(self._item.pixmap_item)
        self._ui.graphicsView.setScene(self._scene)

        self._anim = Animation(self._item, b'pos', self)
        self._anim.setEasingCurve(QEasingCurve.Type.OutBounce)
        self._ui.easingCurvePicker.setCurrentRow(0)

        self.start_animation()

    def create_curve_icons(self):
        pix = QPixmap(self._iconSize)

        gradient = QLinearGradient(0, 0, 0, self._iconSize.height())
        gradient.setColorAt(0.0, QColor(240, 240, 240))
        gradient.setColorAt(1.0, QColor(224, 224, 224))

        brush = QBrush(gradient)

        curve_count = QEasingCurve.Type.Custom.value
        curve_types = [(f"QEasingCurve.{e.name}", e)
                       for e in QEasingCurve.Type if e.value < curve_count]

        with QPainter(pix) as painter:

            for curve_name, curve_type in curve_types:
                painter.fillRect(QRect(QPoint(0, 0), self._iconSize), brush)
                curve = createEasingCurve(curve_type)

                painter.setPen(QColor(0, 0, 255, 64))
                x_axis = self._iconSize.height() / 1.5
                y_axis = self._iconSize.width() / 3.0
                painter.drawLine(0, x_axis, self._iconSize.width(), x_axis)
                painter.drawLine(y_axis, 0, y_axis, self._iconSize.height())

                curve_scale = self._iconSize.height() / 2.0

                painter.setPen(Qt.PenStyle.NoPen)

                # Start point.
                painter.setBrush(Qt.GlobalColor.red)
                start = QPoint(y_axis, x_axis - curve_scale * curve.valueForProgress(0))
                painter.drawRect(start.x() - 1, start.y() - 1, 3, 3)

                # End point.
                painter.setBrush(Qt.GlobalColor.blue)
                end = QPoint(y_axis + curve_scale,
                             x_axis - curve_scale * curve.valueForProgress(1))
                painter.drawRect(end.x() - 1, end.y() - 1, 3, 3)

                curve_path = QPainterPath()
                curve_path.moveTo(QPointF(start))
                t = 0.0
                while t <= 1.0:
                    to = QPointF(y_axis + curve_scale * t,
                                 x_axis - curve_scale * curve.valueForProgress(t))
                    curve_path.lineTo(to)
                    t += 1.0 / curve_scale

                painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
                painter.strokePath(curve_path, QColor(32, 32, 32))
                painter.setRenderHint(QPainter.RenderHint.Antialiasing, False)

                item = QListWidgetItem()
                item.setIcon(QIcon(pix))
                item.setText(curve_name)
                self._ui.easingCurvePicker.addItem(item)

    def start_animation(self):
        self._anim.setStartValue(QPointF(0, 0))
        self._anim.setEndValue(QPointF(100, 100))
        self._anim.setDuration(2000)
        self._anim.setLoopCount(-1)
        self._anim.start()

    def curve_changed(self, row):
        curve_type = QEasingCurve.Type(row)
        self._anim.setEasingCurve(createEasingCurve(curve_type))
        self._anim.setCurrentTime(0)

        is_elastic = (curve_type.value >= QEasingCurve.Type.InElastic.value
                      and curve_type.value <= QEasingCurve.Type.OutInElastic.value)
        is_bounce = (curve_type.value >= QEasingCurve.Type.InBounce.value
                     and curve_type.value <= QEasingCurve.Type.OutInBounce.value)

        self._ui.periodSpinBox.setEnabled(is_elastic)
        self._ui.amplitudeSpinBox.setEnabled(is_elastic or is_bounce)
        overshoot = (curve_type.value >= QEasingCurve.Type.InBack.value
                     and curve_type.value <= QEasingCurve.Type.OutInBack.value)
        self._ui.overshootSpinBox.setEnabled(overshoot)

    def path_changed(self, index):
        self._anim.set_path_type(index)

    def period_changed(self, value):
        curve = self._anim.easingCurve()
        curve.setPeriod(value)
        self._anim.setEasingCurve(curve)

    def amplitude_changed(self, value):
        curve = self._anim.easingCurve()
        curve.setAmplitude(value)
        self._anim.setEasingCurve(curve)

    def overshoot_changed(self, value):
        curve = self._anim.easingCurve()
        curve.setOvershoot(value)
        self._anim.setEasingCurve(curve)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = Window()
    w.resize(600, 600)
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit test for QPixelFormat'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtCore import QSize, Qt
from PySide6.QtGui import QColor, QImage, QPixelFormat, qPixelFormatRgba


class QPixelFormatTest(UsesQApplication):
    def test(self):
        image = QImage(QSize(200, 200), QImage.Format.Format_ARGB32)
        image.fill(QColor(Qt.GlobalColor.red))
        pixelFormat = image.pixelFormat()
        print(pixelFormat.greenSize())
        self.assertEqual(pixelFormat.alphaSize(), 8)
        self.assertEqual(pixelFormat.redSize(), 8)
        self.assertEqual(pixelFormat.greenSize(), 8)
        self.assertEqual(pixelFormat.blueSize(), 8)
        self.assertEqual(pixelFormat.bitsPerPixel(), 32)

    def testHelpers(self):
        format = qPixelFormatRgba(8, 8, 8, 8, QPixelFormat.AlphaUsage.UsesAlpha,
                                  QPixelFormat.AlphaPosition.AtBeginning,
                                  QPixelFormat.AlphaPremultiplied.Premultiplied,
                                  QPixelFormat.TypeInterpretation.UnsignedByte)
        self.assertEqual(format.redSize(), 8)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QObject.sender()'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QObject, QTimer, Signal, Slot
from helper.usesqapplication import UsesQApplication

import samenamesender


class ExtQTimer(QTimer):
    def __init__(self):
        super().__init__()


class Sender(QObject):
    foo = Signal()


class SameNameSender(samenamesender.SameNameSender):
    ''' Test sender class for SameNameSenderTest (PYSIDE-3201).'''
    signal3 = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.signal1.connect(self.slot2)
        self.signal2.connect(self.slot2)
        self.signal3.connect(self.slot3)
        self.slot3_invoked = False

    @Slot()
    def slot1(self):
        pass

    @Slot()
    def slot2(self):
        pass

    @Slot()
    def slot3(self):
        self.slot3_invoked = True


class Receiver(QObject):
    def __init__(self):
        super().__init__()
        self.the_sender = None

    def callback(self):
        self.the_sender = self.sender()
        if QCoreApplication.instance():
            QCoreApplication.instance().exit()


class ResultReceiver(QObject):
    def __init__(self):
        super().__init__()

    @Slot(result=int)
    def slotWithResult(self):
        return 3


class ObjectSenderTest(unittest.TestCase):
    '''Test case for QObject.sender() method.'''

    def testSenderPythonSignal(self):
        sender = Sender()
        recv = Receiver()
        sender.foo.connect(recv.callback)
        sender.foo.emit()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderCheckOnReceiverTest(unittest.TestCase):
    '''Test case for QObject.sender() method, this one tests the equality on the Receiver object.'''

    def testSenderPythonSignal(self):
        sender = Sender()
        recv = Receiver()
        sender.foo.connect(recv.callback)
        sender.foo.emit()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderWithQAppTest(UsesQApplication):
    '''Test case for QObject.sender() method with QApplication.'''

    def testSenderCppSignal(self):
        sender = QTimer()
        sender.setObjectName('foo')
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)

    def testSenderCppSignalSingleShotTimer(self):
        recv = Receiver()
        QTimer.singleShot(10, recv.callback)
        self.app.exec()
        self.assertTrue(isinstance(recv.the_sender, QObject))

    def testSenderCppSignalSingleShotTimerWithContext(self):
        recv = Receiver()
        QTimer.singleShot(10, recv, recv.callback)
        self.app.exec()
        self.assertTrue(isinstance(recv.the_sender, QObject))

    def testSenderCppSignalWithPythonExtendedClass(self):
        sender = ExtQTimer()
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)


class ObjectSenderWithQAppCheckOnReceiverTest(UsesQApplication):
    '''Test case for QObject.sender() method with QApplication.'''

    def testSenderCppSignal(self):
        sender = QTimer()
        sender.setObjectName('foo')
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)

    def testSenderCppSignalWithPythonExtendedClass(self):
        sender = ExtQTimer()
        recv = Receiver()
        sender.timeout.connect(recv.callback)
        sender.start(10)
        self.app.exec()
        self.assertEqual(sender, recv.the_sender)


class SameNameSenderTest(UsesQApplication):
    '''PYSIDE-3201: Test whether the meta object system is confused by identical
       class names.'''
    def test(self):
        sender = SameNameSender()
        sender.signal1.emit()
        sender.signal2.emit()
        sender.signal3.emit()
        self.assertTrue(sender.slot3_invoked)


class ResultSlotTest(UsesQApplication):
    '''PYSIDE-3266: Test that a slot declaring a result type does not cause crashes
       in signal connections due to args[0] == 0.'''
    def test(self):
        sender = Sender()
        recv = ResultReceiver()
        sender.foo.connect(recv.slotWithResult)
        sender.foo.emit()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QTextCursor
from PySide6.QtWidgets import QLayout, QGraphicsLayout, QGraphicsLayoutItem

from helper.usesqapplication import UsesQApplication


class Layout(QLayout):
    def __init__(self):
        super().__init__()


class GraphicsLayout(QGraphicsLayout):
    def __init__(self):
        super().__init__()


class GraphicsLayoutItem(QGraphicsLayoutItem):
    def __init__(self):
        super().__init__()


class ReturnsQuadruplesOfNumbers(UsesQApplication):
    def compareTuples(self, ta, tb):
        for va, vb in zip(ta, tb):
            if round(va) != round(vb):
                return False
        return True

    def testQGraphicsLayoutGetContentsMargins(self):
        obj = GraphicsLayout()
        values = (10.0, 20.0, 30.0, 40.0)
        obj.setContentsMargins(*values)
        self.assertTrue(self.compareTuples(obj.getContentsMargins(), values))

    def testQGraphicsLayoutItemGetContentsMargins(self):
        obj = GraphicsLayoutItem()
        self.assertTrue(self.compareTuples(obj.getContentsMargins(), (0.0, 0.0, 0.0, 0.0)))

    def testQLayoutGetContentsMargins(self):
        obj = Layout()
        values = (10, 20, 30, 40)
        obj.setContentsMargins(*values)
        self.assertTrue(self.compareTuples(obj.getContentsMargins(), values))

    def testQTextCursorSelectedTableCells(self):
        obj = QTextCursor()
        self.assertEqual(obj.selectedTableCells(), (-1, -1, -1, -1))


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from testbinding import getHiddenObject


class TestBug1016 (unittest.TestCase):

    def testIt(self):
        obj = getHiddenObject()
        self.assertEqual(obj.callMe(), None)
        self.assertTrue(obj.wasCalled())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from sample import ObjectType


class TestTypeDestructorDoubleFree(unittest.TestCase):
    def testTypeDestructorDoubleFree(self):
        '''Causes the type destructors of two derived classes to be called.'''
        def scope():
            class ExtObj1(ObjectType):
                def __init__(self):
                    ObjectType.__init__(self)
            obj = ExtObj1()
            child = ObjectType(parent=obj)
            self.assertEqual(obj.takeChild(child), child)

            class ExtObj2(ObjectType):
                def __init__(self):
                    ObjectType.__init__(self)

            obj = ExtObj2()
            child = ObjectType(parent=obj)
            self.assertEqual(obj.takeChild(child), child)
        scope()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt, QObject
from PySide6.QtWidgets import (QComboBox, QGraphicsScene,
                               QGraphicsRectItem)

from helper.usesqapplication import UsesQApplication


class MyDiagram(QGraphicsScene):
    pass


class MyItem(QGraphicsRectItem):
    def itemChange(self, change, value):
        return value


class Sequence:
    # Having the __getitem__ method on a class transform the Python
    # type to a PySequence.
    # Before the patch: aa75437f9119d997dd290471ac3e2cc88ca88bf1
    # "Fix QVariant conversions when using PySequences"
    # one could not use an object from this class, because internally
    # we were requiring that the PySequence was finite.
    def __getitem__(self, key):
        raise IndexError()


class QGraphicsSceneOnQVariantTest(UsesQApplication):
    """Test storage ot QGraphicsScene into QVariants"""
    def setUp(self):
        super(QGraphicsSceneOnQVariantTest, self).setUp()
        self.s = MyDiagram()
        self.i = MyItem()
        self.combo = QComboBox()

    def tearDown(self):
        del self.s
        del self.i
        del self.combo
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(QGraphicsSceneOnQVariantTest, self).tearDown()

    def testIt(self):
        self.s.addItem(self.i)
        self.assertEqual(len(self.s.items()), 1)

    def testSequence(self):
        # PYSIDE-641
        self.combo.addItem("test", userData=Sequence())
        self.assertTrue(isinstance(self.combo.itemData(0), Sequence))


class QVariantConversionTest(UsesQApplication):
    """
    Tests conversion from QVariant to supported type held by QVariant
    """
    def setUp(self):
        super(QVariantConversionTest, self).setUp()
        self.obj = QObject()

    def tearDown(self):
        del self.obj
        super(QVariantConversionTest, self).tearDown()

    def testEnum(self):
        """
        PYSIDE-1798: Test enum is obtained correctly when return through QVariant
        """
        self.obj.setProperty("test", Qt.PenStyle.SolidLine)
        self.assertTrue(isinstance(self.obj.property("test"), Qt.PenStyle))
        self.assertEqual(self.obj.property("test"), Qt.PenStyle.SolidLine)

    def testString(self):
        self.obj.setProperty("test", "test")
        self.assertEqual(self.obj.property("test"), "test")
        self.assertTrue(isinstance(self.obj.property("test"), str))

    def testBytes(self):
        byte_message = bytes("test", 'utf-8')
        self.obj.setProperty("test", byte_message)
        self.assertEqual(self.obj.property("test"), byte_message)
        self.assertTrue(isinstance(self.obj.property("test"), bytes))

    def testBasicTypes(self):
        #bool
        self.obj.setProperty("test", True)
        self.assertEqual(self.obj.property("test"), True)
        self.assertTrue(isinstance(self.obj.property("test"), bool))
        #long
        self.obj.setProperty("test", 2)
        self.assertEqual(self.obj.property("test"), 2)
        self.assertTrue(isinstance(self.obj.property("test"), int))
        #float
        self.obj.setProperty("test", 2.5)
        self.assertEqual(self.obj.property("test"), 2.5)
        self.assertTrue(isinstance(self.obj.property("test"), float))
        #None
        self.obj.setProperty("test", None)
        self.assertEqual(self.obj.property("test"), None)

    def testContainerTypes(self):
        #list
        self.obj.setProperty("test", [1, 2, 3])
        self.assertEqual(self.obj.property("test"), [1, 2, 3])
        self.assertTrue(isinstance(self.obj.property("test"), list))
        #dict
        self.obj.setProperty("test", {1: "one"})
        self.assertEqual(self.obj.property("test"), {1: "one"})
        self.assertTrue(isinstance(self.obj.property("test"), dict))

    def testPyObject(self):
        class Test:
            pass
        test = Test()
        self.obj.setProperty("test", test)
        self.assertEqual(self.obj.property("test"), test)
        self.assertTrue(isinstance(self.obj.property("test"), Test))

    def testQMetaPropertyWrite(self):
        combo_box = QComboBox()
        meta_obj = combo_box.metaObject()
        i = meta_obj.indexOfProperty("sizeAdjustPolicy")
        success = meta_obj.property(i).write(combo_box, QComboBox.SizeAdjustPolicy.AdjustToContents)
        self.assertTrue(success)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the
   qml/examples/qml/tutorials/extending-qml-advanced/advanced4-Grouped-properties example
   from Qt v6.x"""

from pathlib import Path
import sys

from PySide6.QtCore import QCoreApplication
from PySide6.QtQml import QQmlComponent, QQmlEngine

from person import Boy, Girl  # noqa: F401
from birthdayparty import BirthdayParty  # noqa: F401


if __name__ == '__main__':
    app = QCoreApplication(sys.argv)
    engine = QQmlEngine()
    engine.addImportPath(Path(__file__).parent)
    component = QQmlComponent(engine)
    component.loadFromModule("People", "Main")

    party = component.create()
    if not party:
        print(component.errors())
        del engine
        sys.exit(-1)
    host = party.host
    print(f"{host.name} is having a birthday!")
    if isinstance(host, Boy):
        print("He is inviting:")
    else:
        print("She is inviting:")
    best_shoe = None
    for g in range(party.guestCount()):
        guest = party.guest(g)
        name = guest.name
        print(f"    {name}")
        if not best_shoe or best_shoe.shoe.price < guest.shoe.price:
            best_shoe = guest
    if best_shoe:
        print(f"{best_shoe.name} is wearing the best shoes!")
    del engine
    sys.exit(0)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test the QMenu.addAction() method'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QMenu
from helper.usesqapplication import UsesQApplication


class QMenuAddAction(UsesQApplication):

    def openFile(self, *args):
        self.arg = args

    def testQMenuAddAction(self):
        fileMenu = QMenu("&File")

        addNewAction = fileMenu.addAction("&Open...", self.openFile)
        addNewAction.trigger()
        self.assertEqual(self.arg, ())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QSharedMemory'''

import os
import subprocess
import sys
import unittest

from pathlib import Path
FILE = Path(__file__).resolve()
sys.path.append(os.fspath(FILE.parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QSharedMemory, QRandomGenerator, qVersion
from qsharedmemory_client import read_string


TEST_STRING = 'ABCD'


def run(cmd):
    # FIXME Python 3.7: Use subprocess.run()
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=False,
                            universal_newlines=True)
    output, error = proc.communicate()
    proc.wait()
    return_code = proc.returncode
    return (return_code, output, error)


class QSharedMemoryTest(unittest.TestCase):

    def setUp(self):
        r = QRandomGenerator.global_().bounded(1000)
        v = qVersion()
        self._name = f"pyside{v}_test_{r}"
        print(self._name)
        self._shared_memory = QSharedMemory(self._name)

    def tearDown(self):
        if self._shared_memory.isAttached():
            self._shared_memory.detach()

    def test(self):
        # Create and write
        self.assertTrue(self._shared_memory.create(1024, QSharedMemory.ReadWrite))
        self.assertTrue(self._shared_memory.lock())
        mv = memoryview(self._shared_memory.data())
        for idx, c in enumerate(TEST_STRING + chr(0)):
            mv[idx] = ord(c)
        mv = None
        self.assertTrue(self._shared_memory.unlock())

        # Read
        self.assertTrue(self._shared_memory.lock())
        self.assertEqual(read_string(self._shared_memory), TEST_STRING)
        self.assertTrue(self._shared_memory.unlock())

        # Run a subprocess and let it read
        client = FILE.parent / 'qsharedmemory_client.py'
        returncode, output, error = run([sys.executable, client, self._name])
        if error:
            print(error, file=sys.stderr)
        self.assertEqual(returncode, 0)
        self.assertEqual(output, TEST_STRING)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QMimeDatabase'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QMimeDatabase, QLocale


class QMimeDatabaseTest(unittest.TestCase):
    def testMimeTypeForName(self):
        db = QMimeDatabase()

        s0 = db.mimeTypeForName("application/x-zerosize")
        self.assertTrue(s0.isValid())
        self.assertEqual(s0.name(), "application/x-zerosize")
        self.assertTrue(s0.comment())

        s0Again = db.mimeTypeForName("application/x-zerosize")
        self.assertEqual(s0Again.name(), s0.name())

        s1 = db.mimeTypeForName("text/plain")
        self.assertTrue(s1.isValid())
        self.assertEqual(s1.name(), "text/plain")

        # Removed because of the move of to the Tika mimetypes.
        # krita = db.mimeTypeForName("application/x-krita")
        # self.assertTrue(krita.isValid())

        rdf = db.mimeTypeForName("application/rdf+xml")
        self.assertTrue(rdf.isValid())
        self.assertEqual(rdf.name(), "application/rdf+xml")
        self.assertTrue(rdf.comment())
        if "en" in QLocale().name():
            self.assertTrue(rdf.comment() in ("RDF file", "XML syntax for RDF graphs"))

        bzip2 = db.mimeTypeForName("application/x-bzip2")
        self.assertTrue(bzip2.isValid())
        self.assertTrue(bzip2.comment())

        defaultMime = db.mimeTypeForName("application/octet-stream")
        self.assertTrue(defaultMime.isValid())
        self.assertEqual(defaultMime.name(), "application/octet-stream")
        self.assertTrue(defaultMime.isDefault())

        doesNotExist = db.mimeTypeForName("foobar/x-doesnot-exist")
        self.assertTrue(not doesNotExist.isValid())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtAsyncio'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import asyncio
import random
import time
from PySide6.QtAsyncio import QAsyncioEventLoopPolicy


class QAsyncioTestCaseQueues(unittest.TestCase):

    async def produce(self, output, queue):
        for _ in range(random.randint(0, 2)):
            await asyncio.sleep(random.random())
            await queue.put(self.i)
            output += f"{self.i} added to queue\n"
            self.i += 1

    async def consume(self, output, queue):
        while True:
            await asyncio.sleep(random.random())
            i = await queue.get()
            output += f"{i} pulled from queue\n"
            queue.task_done()

    async def main(self, output1, output2, num_producers, num_consumers):
        self.i = 0
        queue = asyncio.Queue()
        producers = [
            asyncio.create_task(self.produce(output1, queue)) for _ in range(num_producers)]
        consumers = [
            asyncio.create_task(self.consume(output2, queue)) for _ in range(num_consumers)]
        await asyncio.gather(*producers)
        await queue.join()
        for consumer in consumers:
            consumer.cancel()

    def test_queues(self):
        args = [(2, 3), (2, 1)]
        for arg in args:
            outputs_expected1 = []
            outputs_expected2 = []
            outputs_real1 = []
            outputs_real2 = []

            # Run the code without QAsyncioEventLoopPolicy
            random.seed(17)
            start = time.perf_counter()
            asyncio.set_event_loop_policy(asyncio.DefaultEventLoopPolicy())
            asyncio.run(self.main(outputs_expected1, outputs_expected2, *arg))
            end_expected = time.perf_counter() - start

            # Run the code with QAsyncioEventLoopPolicy and QtEventLoop
            random.seed(17)
            start = time.perf_counter()
            asyncio.set_event_loop_policy(QAsyncioEventLoopPolicy())
            asyncio.run(self.main(outputs_real1, outputs_real2, *arg))
            end_real = time.perf_counter() - start

            self.assertEqual(outputs_expected1, outputs_real1)
            self.assertEqual(outputs_expected2, outputs_real2)
            self.assertAlmostEqual(end_expected, end_real, delta=1)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""Test grouped properties (PYSIDE-1836)."""

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import (QCoreApplication, QUrl, QObject, Property)  # noqa: E402
from PySide6.QtQml import (QQmlComponent, QQmlEngine, QmlAnonymous, QmlElement)  # noqa: E402


QML_IMPORT_NAME = "grouped"
QML_IMPORT_MAJOR_VERSION = 1


@QmlAnonymous
class ShoeDescription(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._brand = ""
        self._size = 0
        self._price = 0

    @Property(str)
    def brand(self):
        return self._brand

    @brand.setter
    def brand(self, b):
        self._brand = b

    @Property(int)
    def size(self):
        return self._size

    @size.setter
    def size(self, s):
        self._size = s

    @Property(int)
    def price(self):
        return self._price

    @price.setter
    def price(self, p):
        self._price = p


@QmlElement
class Person(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._name = ""
        self._shoe = ShoeDescription()

    @Property(str)
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n

    @Property(ShoeDescription)
    def shoe(self):
        return self._shoe


def component_error(component):
    result = ""
    for e in component.errors():
        if result:
            result += "\n"
        result += str(e)
    return result


class TestQmlGroupedProperties(unittest.TestCase):
    def testIt(self):
        app = QCoreApplication(sys.argv)  # noqa: F841
        file = Path(__file__).resolve().parent / "groupedproperty.qml"
        url = QUrl.fromLocalFile(file)
        engine = QQmlEngine()
        component = QQmlComponent(engine, url)
        person = component.create()
        self.assertTrue(person, component_error(component))

        # Check the meta type of the property
        meta_object = person.metaObject()
        index = meta_object.indexOfProperty("shoe")
        self.assertTrue(index > 0)
        meta_property = meta_object.property(index)
        meta_type = meta_property.metaType()
        self.assertTrue(meta_type.isValid())

        # Check the values
        self.assertEqual(person.shoe.brand, "Bikey")
        self.assertEqual(person.shoe.price, 90)
        self.assertEqual(person.shoe.size, 12)

        del engine


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from math import cos, degrees, sqrt

from PySide6.QtCore import QObject, Signal, Slot, Qt
from PySide6.QtGui import QVector3D
from PySide6.QtDataVisualization import (QAbstract3DGraph, QAbstract3DSeries,
                                         QScatterDataItem, QScatterDataProxy,
                                         QScatter3DSeries, Q3DCamera,
                                         Q3DTheme)

from axesinputhandler import AxesInputHandler


NUMBER_OF_ITEMS = 10000
CURVE_DIVIDER = 7.5
LOWER_NUMBER_OF_ITEMS = 900
LOWER_CURVE_DIVIDER = 0.75


class ScatterDataModifier(QObject):

    backgroundEnabledChanged = Signal(bool)
    gridEnabledChanged = Signal(bool)
    shadowQualityChanged = Signal(int)

    def __init__(self, scatter, parent):
        super().__init__(parent)

        self._graph = scatter

        self._style = QAbstract3DSeries.MeshSphere
        self._smooth = True
        self._inputHandler = AxesInputHandler(scatter)
        self._autoAdjust = True
        self._itemCount = LOWER_NUMBER_OF_ITEMS
        self._CURVE_DIVIDER = LOWER_CURVE_DIVIDER
        self._inputHandler = AxesInputHandler(scatter)

        self._graph.activeTheme().setType(Q3DTheme.ThemeStoneMoss)
        self._graph.setShadowQuality(QAbstract3DGraph.ShadowQualitySoftHigh)
        self._graph.scene().activeCamera().setCameraPreset(Q3DCamera.CameraPresetFront)
        self._graph.scene().activeCamera().setZoomLevel(80.0)

        self._proxy = QScatterDataProxy()
        self._series = QScatter3DSeries(self._proxy)
        self._series.setItemLabelFormat("@xTitle: @xLabel @yTitle: @yLabel @zTitle: @zLabel")
        self._series.setMeshSmooth(self._smooth)
        self._graph.addSeries(self._series)

        # Give ownership of the handler to the graph and make it the active
        # handler
        self._graph.setActiveInputHandler(self._inputHandler)

        # Give our axes to the input handler
        self._inputHandler.setAxes(self._graph.axisX(), self._graph.axisZ(),
                                   self._graph.axisY())

        self.addData()

    def addData(self):
        # Configure the axes according to the data
        self._graph.axisX().setTitle("X")
        self._graph.axisY().setTitle("Y")
        self._graph.axisZ().setTitle("Z")

        dataArray = []
        limit = int(sqrt(self._itemCount) / 2.0)
        for i in range(-limit, limit):
            for j in range(-limit, limit):
                x = float(i) + 0.5
                y = cos(degrees(float(i * j) / self._CURVE_DIVIDER))
                z = float(j) + 0.5
                dataArray.append(QScatterDataItem(QVector3D(x, y, z)))

        self._graph.seriesList()[0].dataProxy().resetArray(dataArray)

    @Slot(int)
    def changeStyle(self, style):
        comboBox = self.sender()
        if comboBox:
            self._style = comboBox.itemData(style)
            if self._graph.seriesList():
                self._graph.seriesList()[0].setMesh(self._style)

    @Slot(int)
    def setSmoothDots(self, smooth):
        self._smooth = smooth == Qt.Checked.value
        series = self._graph.seriesList()[0]
        series.setMeshSmooth(self._smooth)

    @Slot(int)
    def changeTheme(self, theme):
        currentTheme = self._graph.activeTheme()
        currentTheme.setType(Q3DTheme.Theme(theme))
        self.backgroundEnabledChanged.emit(currentTheme.isBackgroundEnabled())
        self.gridEnabledChanged.emit(currentTheme.isGridEnabled())

    @Slot()
    def changePresetCamera(self):
        preset = Q3DCamera.CameraPresetFrontLow.value

        camera = self._graph.scene().activeCamera()
        camera.setCameraPreset(Q3DCamera.CameraPreset(preset))

        preset += 1
        if preset > Q3DCamera.CameraPresetDirectlyBelow.value:
            preset = Q3DCamera.CameraPresetFrontLow.value

    @Slot(QAbstract3DGraph.ShadowQuality)
    def shadowQualityUpdatedByVisual(self, sq):
        self.shadowQualityChanged.emit(sq.value)

    @Slot(int)
    def changeShadowQuality(self, quality):
        sq = QAbstract3DGraph.ShadowQuality(quality)
        self._graph.setShadowQuality(sq)

    @Slot(int)
    def setBackgroundEnabled(self, enabled):
        self._graph.activeTheme().setBackgroundEnabled(enabled == Qt.Checked.value)

    @Slot(int)
    def setGridEnabled(self, enabled):
        self._graph.activeTheme().setGridEnabled(enabled == Qt.Checked.value)

    @Slot()
    def toggleItemCount(self):
        if self._itemCount == NUMBER_OF_ITEMS:
            self._itemCount = LOWER_NUMBER_OF_ITEMS
            self._CURVE_DIVIDER = LOWER_CURVE_DIVIDER
        else:
            self._itemCount = NUMBER_OF_ITEMS
            self._CURVE_DIVIDER = CURVE_DIVIDER

        self._graph.seriesList()[0].dataProxy().resetArray([])
        self.addData()

    @Slot()
    def toggleRanges(self):
        if not self._autoAdjust:
            self._graph.axisX().setAutoAdjustRange(True)
            self._graph.axisZ().setAutoAdjustRange(True)
            self._inputHandler.setDragSpeedModifier(1.5)
            self._autoAdjust = True
        else:
            self._graph.axisX().setRange(-10.0, 10.0)
            self._graph.axisZ().setRange(-10.0, 10.0)
            self._inputHandler.setDragSpeedModifier(15.0)
            self._autoAdjust = False



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from PySide6.QtCore import QObject, Signal, Slot, SIGNAL, SLOT
from testbinding import TestObject


class Sender(QObject):
    bar = Signal()


class Receiver(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.called = 0

    @Slot()
    def receiver(self):
        self.called += 1


class TestDisconnect(unittest.TestCase):
    def theSlot1(self):
        self.called1 = True

    def theSlot2(self):
        self.called2 = True

    def testIt(self):
        self.called1 = False
        self.called2 = False
        s = Sender()
        s.bar.connect(self.theSlot1)
        s.bar.connect(self.theSlot2)
        s.bar.emit()
        self.assertTrue(self.called1)
        self.assertTrue(self.called2)

        self.called1 = False
        self.called2 = False
        self.assertTrue(s.bar.disconnect())  # Disconnect sender
        s.bar.emit()
        self.assertFalse(self.called1)
        self.assertFalse(self.called2)

    def testCallable(self):
        s = Sender()
        r = Receiver()
        s.bar.connect(r.receiver)
        s.bar.emit()
        self.assertEqual(r.called, 1)
        self.assertTrue(s.bar.disconnect(r.receiver))
        s.bar.emit()
        self.assertEqual(r.called, 1)

    def testStringBased(self):
        s = Sender()
        r = Receiver()
        QObject.connect(s, SIGNAL("bar()"), r, SLOT("receiver()"))
        s.bar.emit()
        self.assertEqual(r.called, 1)
        self.assertTrue(QObject.disconnect(s, SIGNAL("bar()"), r, SLOT("receiver()")))
        s.bar.emit()
        self.assertEqual(r.called, 1)

    def testMixStringBasedCallable(self):
        """PYSIDE-3020, Disconnect a string-based connection by passing a callable."""
        s = Sender()
        r = Receiver()
        QObject.connect(s, SIGNAL("bar()"), r, SLOT("receiver()"))
        s.bar.emit()
        self.assertEqual(r.called, 1)
        self.assertTrue(s.bar.disconnect(r.receiver))
        s.bar.emit()
        self.assertEqual(r.called, 1)

    def testMixCallableStringBased(self):
        """PYSIDE-3020, test vice versa."""
        s = Sender()
        r = Receiver()
        s.bar.connect(r.receiver)
        s.bar.emit()
        self.assertEqual(r.called, 1)
        self.assertTrue(QObject.disconnect(s, SIGNAL("bar()"), r, SLOT("receiver()")))
        s.bar.emit()
        self.assertEqual(r.called, 1)

    def testMetaMethod(self):
        s = Sender()
        r = Receiver()
        sender_mo = s.metaObject()
        signal_index = sender_mo.indexOfMethod("bar()")
        self.assertTrue(signal_index != -1)
        signal_method = sender_mo.method(signal_index)
        receiver_mo = r.metaObject()
        slot_index = receiver_mo.indexOfMethod("receiver()")
        self.assertTrue(slot_index != -1)
        slot_method = receiver_mo.method(slot_index)
        conn_id = QObject.connect(s, signal_method, r, slot_method)
        self.assertTrue(conn_id)
        s.bar.emit()
        self.assertEqual(r.called, 1)
        self.assertTrue(QObject.disconnect(s, signal_method, r, slot_method))
        s.bar.emit()
        self.assertEqual(r.called, 1)

    def testDuringCallback(self):
        """ Test to see if the C++ object for a connection is accessed after the
        method returns.  This causes a segfault if the memory that was used by the
        C++ object has been reused. """

        self.called = False
        obj = TestObject(0)

        def callback():
            obj.signalWithDefaultValue.disconnect(callback)

            # Connect more callbacks to try to overwrite memory
            for i in range(1000):
                obj.signalWithDefaultValue.connect(lambda: None)

            self.called = True

            # A non-None return value is needed
            return True
        obj.signalWithDefaultValue.connect(callback)
        obj.signalWithDefaultValue.emit()
        self.assertTrue(self.called)

    def testMultipleConnections(self):
        """PYSIDE-3190: Signal.disconnect() should use the QMetaObject
           code to disconnect, disconnecting multiple connections."""
        s = Sender()
        r = Receiver()
        s.bar.connect(r.receiver)
        s.bar.connect(r.receiver)
        s.bar.emit()
        print(r.called)
        self.assertEqual(r.called, 2)
        s.bar.disconnect(r.receiver)
        s.bar.emit()
        self.assertEqual(r.called, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import (QCoreApplication, QObject, QParallelAnimationGroup,
                            QTimer, SIGNAL)
from PySide6.QtStateMachine import QEventTransition, QFinalState, QState, QStateMachine


REF_COUNT_DELTA = 2 if sys.version_info >= (3, 14) else 1


def addStates(transition):
    sx = QState()
    sy = QState()
    transition.setTargetStates([sx, sy])


def addAnimation(transition):
    animation = QParallelAnimationGroup()
    transition.addAnimation(animation)


class QAbstractTransitionTest(unittest.TestCase):

    def testBasic(self):
        app = QCoreApplication([])

        o = QObject()
        o.setProperty("text", "INdT")

        machine = QStateMachine()
        s1 = QState()
        s1.assignProperty(o, "text", "Rocks")

        s2 = QFinalState()
        t = s1.addTransition(o, SIGNAL("change()"), s2)

        self.assertEqual(t.targetStates(), [s2])

        addStates(t)
        self.assertEqual(len(t.targetStates()), 2)

        animation = QParallelAnimationGroup()
        t.addAnimation(animation)

        self.assertEqual(t.animations(), [animation])

        addAnimation(t)
        self.assertEqual(t.animations()[0].parent(), None)

        machine.addState(s1)
        machine.addState(s2)
        machine.setInitialState(s1)
        machine.start()

        QTimer.singleShot(100, app.quit)
        app.exec()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountOfTargetState(self):
        transition = QEventTransition()
        state1 = QState()
        refcount1 = sys.getrefcount(state1)

        transition.setTargetState(state1)

        self.assertEqual(transition.targetState(), state1)
        self.assertEqual(sys.getrefcount(transition.targetState()),
                         refcount1 + REF_COUNT_DELTA)

        state2 = QState()
        refcount2 = sys.getrefcount(state2)

        transition.setTargetState(state2)

        self.assertEqual(transition.targetState(), state2)
        self.assertEqual(sys.getrefcount(transition.targetState()),
                         refcount2 + REF_COUNT_DELTA)
        self.assertEqual(sys.getrefcount(state1), refcount1)

        del transition

        self.assertEqual(sys.getrefcount(state2), refcount2)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountOfTargetStates(self):
        transition = QEventTransition()
        state1 = QState()
        state2 = QState()
        states = [state1, state2]
        refcount1 = sys.getrefcount(state1)
        refcount2 = sys.getrefcount(state2)

        transition.setTargetStates(states)

        self.assertEqual(transition.targetStates(), states)
        self.assertEqual(transition.targetState(), state1)
        self.assertEqual(sys.getrefcount(transition.targetStates()[0]),
                         refcount1 + REF_COUNT_DELTA)
        self.assertEqual(sys.getrefcount(transition.targetStates()[1]),
                         refcount2 + REF_COUNT_DELTA)

        del states
        del transition

        self.assertEqual(sys.getrefcount(state1), refcount1 - 1)
        self.assertEqual(sys.getrefcount(state2), refcount2 - 1)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountOfTargetStatesAfterSingleTargetState(self):
        transition = QEventTransition()
        state0 = QState()
        refcount0 = sys.getrefcount(state0)

        transition.setTargetState(state0)

        self.assertEqual(transition.targetState(), state0)
        self.assertEqual(sys.getrefcount(transition.targetState()),
                         refcount0 + REF_COUNT_DELTA)

        state1 = QState()
        state2 = QState()
        states = [state1, state2]
        refcount1 = sys.getrefcount(state1)
        refcount2 = sys.getrefcount(state2)

        transition.setTargetStates(states)

        self.assertEqual(sys.getrefcount(state0), refcount0)
        self.assertEqual(transition.targetStates(), states)
        self.assertEqual(transition.targetState(), state1)
        self.assertEqual(sys.getrefcount(transition.targetStates()[0]),
                         refcount1 + REF_COUNT_DELTA)
        self.assertEqual(sys.getrefcount(transition.targetStates()[1]),
                         refcount2 + REF_COUNT_DELTA)

        del states
        del transition

        self.assertEqual(sys.getrefcount(state1), refcount1 - 1)
        self.assertEqual(sys.getrefcount(state2), refcount2 - 1)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountOfTargetStatesBeforeSingleTargetState(self):
        transition = QEventTransition()
        state1 = QState()
        state2 = QState()
        states = [state1, state2]
        refcount1 = sys.getrefcount(state1)
        refcount2 = sys.getrefcount(state2)

        transition.setTargetStates(states)

        self.assertEqual(transition.targetStates(), states)
        self.assertEqual(transition.targetState(), state1)
        self.assertEqual(sys.getrefcount(transition.targetStates()[0]),
                         refcount1 + REF_COUNT_DELTA)
        self.assertEqual(sys.getrefcount(transition.targetStates()[1]),
                         refcount2 + REF_COUNT_DELTA)

        state3 = QState()
        refcount3 = sys.getrefcount(state3)

        transition.setTargetState(state3)

        self.assertEqual(transition.targetState(), state3)
        self.assertEqual(sys.getrefcount(transition.targetState()),
                         refcount3 + REF_COUNT_DELTA)

        del states

        self.assertEqual(sys.getrefcount(state1), refcount1 - 1)
        self.assertEqual(sys.getrefcount(state2), refcount2 - 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QTimer.singleShot'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QTimer
from helper.usesqapplication import UsesQApplication


class TestDeleteLater(UsesQApplication):
    '''Test case for function DeleteLater'''

    def testCase(self):
        o = QObject()
        o.deleteLater()
        del o
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        QTimer.singleShot(100, self.app.quit)
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import ast
import enum
import keyword
import os
import re
import sys
import typing
import warnings
import collections.abc
import abc

from types import SimpleNamespace
from shibokensupport.signature.mapping import (type_map, type_map_tuple, update_mapping,
    namespace, _NotCalled, ResultVariable, ArrayLikeVariable, pyside_modules)  # noqa E:128
from shibokensupport.signature.lib.tool import build_brace_pattern
from shibokensupport.signature import make_snake_case_name

_DEBUG = False
LIST_KEYWORDS = False

"""
parser.py

This module parses the signature text and creates properties for the
signature objects.

PySide has a new function 'CppGenerator::writeSignatureInfo()'
that extracts the gathered information about the function arguments
and defaults as good as it can. But what PySide generates is still
very C-ish and has many constants that Python doesn't understand.

The function 'try_to_guess()' below understands a lot of PySide's
peculiar way to assume local context. If it is able to do the guess,
then the result is inserted into the dict, so the search happens
not again. For everything that is not covered by these automatic
guesses, we provide an entry in 'type_map' that resolves it.

In effect, 'type_map' maps text to real Python objects.
"""


def _get_flag_enum_option():
    from shiboken6 import (__version_info__ as ver,  # noqa F:401
                           __minimum_python_version__ as pyminver,
                           __maximum_python_version__ as pymaxver)

    # PYSIDE-1735: Use the new Enums per default if version is >= 6.4
    #              This decides between delivered vs. dev versions.
    #              When 6.4 is out, the switching mode will be gone.
    flag = ver[:2] >= (6, 4)
    envname = "PYSIDE6_OPTION_PYTHON_ENUM"
    sysname = envname.lower()
    opt = os.environ.get(envname)
    if opt:
        opt = opt.lower()
        if opt in ("yes", "on", "true"):
            flag = True
        elif opt in ("no", "off", "false"):
            flag = False
        else:
            # instead of a simple int() conversion, let's allow for "0xf" or "0b1111"
            try:
                flag = ast.literal_eval(opt)
            except Exception:
                flag = False    # turn a forbidden option into an error
    elif hasattr(sys, sysname):
        opt2 = flag = getattr(sys, sysname)
        if not isinstance(flag, int):
            flag = False        # turn a forbidden option into an error
    p = f"\n    *** Python is at version {'.'.join(map(str, pyminver or (0,)))} now."
    q = f"\n    *** PySide is at version {'.'.join(map(str, ver[:2]))} now."
    # _PepUnicode_AsString: Fix a broken promise
    if pyminver and pyminver >= (3, 10):
        warnings.warn(f"{p} _PepUnicode_AsString can now be replaced by PyUnicode_AsUTF8! ***")
    # PYSIDE-3012: Emit a warning when we may simplify layout.py and pyi_generator.py
    if pyminver and pyminver >= (3, 10):
        warnings.warn(f"{p} layout.py and pyi_generator.py can now remove old code! ***")
    # PYSIDE-1960: Emit a warning when we may remove bufferprocs_py37.(cpp|h)
    if pyminver and pyminver >= (3, 11):
        warnings.warn(f"{p} The files bufferprocs_py37.(cpp|h) should be removed ASAP! ***")
    # PYSIDE-1735: Emit a warning when we should maybe evict forgiveness mode
    if ver[:2] >= (7, 0):
        warnings.warn(f"{q} Please drop Enum forgiveness mode in `mangled_type_getattro` ***")
    # PYSIDE-2404: Emit a warning when we should drop uppercase offset constants
    if ver[:2] >= (7, 0):
        warnings.warn(f"{q} Please drop uppercase type offsets in `copyOffsetEnumStream` ***")
    # normalize the sys attribute
    setattr(sys, sysname, flag)
    os.environ[envname] = str(flag)
    if int(flag) == 0:
        raise RuntimeError(f"Old Enums are no longer supported. int({opt or opt2}) evaluates to 0)")
    return flag


class EnumSelect(enum.Enum):
    # PYSIDE-1735: Here we could save object.value expressions by using IntEnum.
    #              But it is nice to use just an Enum for selecting Enum version.
    OLD = 1
    NEW = 2
    SELECTION = NEW if _get_flag_enum_option() else OLD


def dprint(*args, **kw):
    if _DEBUG:
        import pprint
        for arg in args:
            pprint.pprint(arg)
            sys.stdout.flush()


class ArglistParser:
    def __init__(self):
        regex = build_brace_pattern(level=3, separators=",")
        rec = re.compile(regex, flags=re.VERBOSE)
        self._finditer = rec.finditer

    def parse(self, argstr):
        return list(x.group(1).strip() for x in self._finditer(argstr))


arglistParser = ArglistParser()


def _parse_arglist(argstr):
    return arglistParser.parse(argstr)


def _parse_line(line):
    line_re = r"""
        ((?P<multi> ([0-9]+)) : )?    # the optional multi-index
        (?P<funcname> \w+(\.\w+)*)    # the function name
        \( (?P<arglist> .*?) \)       # the argument list
        ( -> (?P<returntype> .*) )?   # the optional return type
        $
        """
    matches = re.match(line_re, line, re.VERBOSE)
    if not matches:
        raise SystemError("Error parsing line:", repr(line))
    ret = SimpleNamespace(**matches.groupdict())
    # PYSIDE-1095: Handle arbitrary default expressions
    argstr = ret.arglist.replace("->", ".deref.")
    arglist = _parse_arglist(argstr)
    args = []
    for idx, arg in enumerate(arglist):
        tokens = arg.split(":")
        if len(tokens) < 2 and idx == 0 and tokens[0] in ("self", "cls"):
            tokens = 2 * tokens     # "self: self"
        if len(tokens) != 2:
            # This should never happen again (but who knows?)
            raise SystemError(f'Invalid argument "{arg}" in "{line}".')
        name, ann = tokens
        if name in keyword.kwlist:
            if LIST_KEYWORDS:
                print("KEYWORD", ret)
            name = name + "_"
        if "=" in ann:
            ann, default = ann.split("=", 1)
            tup = name, ann, default
        else:
            tup = name, ann
        args.append(tup)
    ret.arglist = args
    multi = ret.multi
    if multi is not None:
        ret.multi = int(multi)
    funcname = ret.funcname
    parts = funcname.split(".")
    if parts[-1] in keyword.kwlist:
        ret.funcname = funcname + "_"
    return vars(ret)


def using_snake_case():
    # Note that this function should stay here where we use snake_case.
    # This function is only meant for creating correct PYI files.
    if "PySide6.QtCore" not in sys.modules:
        return False
    from PySide6.QtCore import QDir
    return hasattr(QDir, "cd_up")


def _handle_instance_fixup(thing):
    """
    Default expressions using instance methods like
        (...,device=QPointingDevice.primaryPointingDevice())
    need extra handling for snake_case. These are:
        QPointingDevice.primaryPointingDevice()
        QInputDevice.primaryKeyboard()
        QKeyCombination.fromCombined(0)
        QSslConfiguration.defaultConfiguration()
    """
    match = re.search(r"\w+\(", thing)
    if not match:
        return thing
    start, stop = match.start(), match.end() - 1
    pre, func, args = thing[:start], thing[start:stop], thing[stop:]
    snake_func = make_snake_case_name(func)
    return f"{pre}{snake_func}{args}"


def make_good_value(thing, valtype):
    # PYSIDE-1019: Handle instance calls (which are really seldom)
    if "(" in thing and using_snake_case():
        thing = _handle_instance_fixup(thing)
    try:
        if thing.endswith("()"):
            thing = f'Default("{thing[:-2]}")'
        else:
            # PYSIDE-1735: Use explicit globals and locals because of a bug in VsCode
            ret = eval(thing, globals(), namespace)
            if valtype and repr(ret).startswith("<"):
                thing = f'Instance("{thing}")'
        return eval(thing, globals(), namespace)
    except Exception:
        pass


def try_to_guess(thing, valtype):
    if "." not in thing and "(" not in thing:
        text = f"{valtype}.{thing}"
        ret = make_good_value(text, valtype)
        if ret is not None:
            return ret
    typewords = valtype.split(".")
    valwords = thing.split(".")
    braceless = valwords[0]    # Yes, not -1. Relevant is the overlapped word.
    if "(" in braceless:
        braceless = braceless[:braceless.index("(")]
    for idx, w in enumerate(typewords):
        if w == braceless:
            text = ".".join(typewords[:idx] + valwords)
            ret = make_good_value(text, valtype)
            if ret is not None:
                return ret
    return None


def get_name(thing):
    if isinstance(thing, type):
        return getattr(thing, "__qualname__", thing.__name__)
    else:
        return thing.__name__


def _resolve_value(thing, valtype, line):
    if thing in ("0", "None") and valtype:
        if valtype.startswith(("PySide6.", "typing.", "collections.abc.")):
            return None
        mapped = type_map.get(valtype)
        # typing.Any: '_SpecialForm' object has no attribute '__name__'
        name = get_name(mapped) if hasattr(mapped, "__name__") else str(mapped)
        thing = f"zero({name})"
        type_map[f"zero({name})"] = None
    if thing in type_map:
        return type_map[thing]
    res = make_good_value(thing, valtype)
    if res is not None:
        type_map[thing] = res
        return res
    res = try_to_guess(thing, valtype) if valtype else None
    if res is not None:
        type_map[thing] = res
        return res
    # Still not found. Look into the imported modules.
    if res := get_from_another_module(thing):
        return res
    warnings.warn(f"""pyside_type_init:_resolve_value

        UNRECOGNIZED:   {thing!r}
        OFFENDING LINE: {line!r}
        """, RuntimeWarning)
    return thing


def _resolve_arraytype(thing, line):
    search = re.search(r"\[(\d*)\]$", thing)
    thing = thing[:search.start()]
    if thing.endswith("]"):
        thing = _resolve_arraytype(thing, line)
    if search.group(1):
        # concrete array, use a tuple
        nelem = int(search.group(1))
        thing = ", ".join([thing] * nelem)
        thing = "Tuple[" + thing + "]"
    else:
        thing = "QList[" + thing + "]"
    return thing


def to_string(thing):
    # This function returns a string that creates the same object.
    # It is absolutely crucial that str(eval(thing)) == str(thing),
    # i.e. it must be an idempotent mapping.
    if isinstance(thing, str):
        return thing
    # PYSIDE-2517: findChild/findChildren type hints:
    # TypeVar doesn't have a __qualname__ attribute,
    # so we fall back to use __name__ before the next condition.
    if isinstance(thing, typing.TypeVar):
        return get_name(thing)
    if hasattr(thing, "__name__") and thing.__module__ not in ("typing", "collections.abc"):
        m = thing.__module__
        dot = "." in str(thing) or m not in (thing.__qualname__, "builtins")
        name = get_name(thing)
        ret = m + "." + name if dot else name
        assert (eval(ret, globals(), namespace))
        return ret
    elif type(thing) in (abc.ABCMeta, type):
        # collections.abc.Sequence without argument is very different from typing.
        # PYSIDE-3147 - type(typing.Union) is a type, not typing._SpecialForm.
        return f"{thing.__module__}.{thing.__name__}"
    # Note: This captures things from the typing module:
    return str(thing)


matrix_pattern = "PySide6.QtGui.QGenericMatrix"


def handle_matrix(arg):
    n, m, typstr = tuple(map(lambda x: x.strip(), arg.split(",")))
    assert typstr == "float"
    result = f"PySide6.QtGui.QMatrix{n}x{m}"
    return eval(result, globals(), namespace)


def get_from_another_module(thing):
    top = thing.split(".", 1)[0] if "." in thing else thing
    for mod_name in pyside_modules:
        mod = sys.modules[mod_name]
        if hasattr(mod, top):
            try:
                res = eval(f"{mod_name}.{thing}", globals(), namespace)
                type_map[thing] = res
                return res
            except AttributeError:
                # Maybe it was another module...
                pass
    return None


def _resolve_type(thing, line, level, var_handler, func_name=None):
    # manual set of 'str' instead of 'bytes'
    if func_name:
        new_thing = (func_name, thing)
        if new_thing in type_map_tuple:
            return type_map_tuple[new_thing]

    # Capture total replacements, first. Happens in
    # "PySide6.QtCore.QCborStreamReader.StringResult[PySide6.QtCore.QByteArray]"
    if thing in type_map:
        return type_map[thing]

    # Now the nested structures are handled.
    if "[" in thing:
        # Special case: Callable[[],
        if thing == "[]":
            return thing
        # handle primitive arrays
        if re.search(r"\[\d*\]$", thing):
            thing = _resolve_arraytype(thing, line)
        # Handle a container return type. (see PYSIDE-921 in cppgenerator.cpp)
        contr, thing = re.match(r"(.*?)\[(.*?)\]$", thing).groups()
        # Special case: Handle the generic matrices.
        if contr == matrix_pattern:
            return handle_matrix(thing)
        contr = var_handler(_resolve_type(contr, line, level + 1, var_handler))
        if isinstance(contr, _NotCalled):
            raise SystemError("Container types must exist:", repr(contr))
        contr = to_string(contr)
        pieces = []
        for part in _parse_arglist(thing):
            part = var_handler(_resolve_type(part, line, level + 1, var_handler))
            if isinstance(part, _NotCalled):
                # fix the tag (i.e. "Missing") by repr
                part = repr(part)
            pieces.append(to_string(part))
        thing = ", ".join(pieces)
        result = f"{contr}[{thing}]"
        # PYSIDE-1538: Make sure that the eval does not crash.
        try:
            return eval(result, globals(), namespace)
        except Exception:
            warnings.warn(f"""pyside_type_init:_resolve_type

                UNRECOGNIZED:   {result!r}
                OFFENDING LINE: {line!r}
                """, RuntimeWarning)
    return _resolve_value(thing, None, line)


def _handle_generic(obj, repl):
    """
    Assign repl if obj is an ArrayLikeVariable

    This is a neat trick. Example:

        obj                     repl        result
        ----------------------  --------    ---------
        ArrayLikeVariable       List        List
        ArrayLikeVariable(str)  List        List[str]
        ArrayLikeVariable       Sequence    Sequence
        ArrayLikeVariable(str)  Sequence    Sequence[str]
    """
    if isinstance(obj, ArrayLikeVariable):
        return repl[obj.type]
    if isinstance(obj, type) and issubclass(obj, ArrayLikeVariable):
        # was "if obj is ArrayLikeVariable"
        return repl
    return obj


def handle_argvar(obj):
    """
    Decide how array-like variables are resolved in arguments

    Currently, the best approximation is types.Sequence.
    We want to change that to types.Iterable in the near future.
    """
    return _handle_generic(obj, collections.abc.Sequence)


def handle_retvar(obj):
    """
    Decide how array-like variables are resolved in results

    This will probably stay typing.List forever.
    """
    return _handle_generic(obj, typing.List)


class Hashabledict(dict):
    def __hash__(self):
        return hash(frozenset(self))


def calculate_props(line):
    parsed = SimpleNamespace(**_parse_line(line.strip()))
    arglist = parsed.arglist
    annotations = {}
    _defaults = []
    for idx, tup in enumerate(arglist):
        name, ann = tup[:2]
        if ann == "...":
            name = "*args" if name.startswith("arg_") else "*" + name
            # copy the patched fields back
            ann = 'nullptr'     # maps to None
            tup = name, ann
            arglist[idx] = tup
        # When the variable name from constructor and the property is the same,
        # one overwrites the other. This is fixed here and the latter is omitted.
        # The property variables added in layout.py
        if name not in annotations:
            annotations[name] = _resolve_type(ann, line, 0, handle_argvar, parsed.funcname)
        if len(tup) == 3:
            default = _resolve_value(tup[2], ann, line)
            # PYSIDE-2846: When creating signatures, the defaults should be hashable.
            #              For that to work, we use `Hashabledict`.
            if type(default) is dict:
                default = Hashabledict(default)
            _defaults.append(default)
    defaults = tuple(_defaults)
    returntype = parsed.returntype
    if isinstance(returntype, str) and returntype.startswith("("):
        # PYSIDE-1588: Simplify the handling of returned tuples for now.
        # Later we might create named tuples, instead.
        returntype = "Tuple"
    # PYSIDE-1383: We need to handle `None` explicitly.
    annotations["return"] = (_resolve_type(returntype, line, 0, handle_retvar)
                             if returntype is not None else None)
    props = SimpleNamespace()
    props.defaults = defaults
    props.kwdefaults = {}
    props.annotations = annotations
    props.varnames = tuple(tup[0] for tup in arglist)
    funcname = parsed.funcname
    shortname = funcname[funcname.rindex(".") + 1:]
    props.name = shortname
    props.fullname = funcname
    props.multi = parsed.multi
    fix_variables(props, line)
    return vars(props)


def fix_variables(props, line):
    annos = props.annotations
    if not any(isinstance(ann, (ResultVariable, ArrayLikeVariable))
               for ann in annos.values()):
        return
    retvar = annos.get("return", None)
    if retvar and isinstance(retvar, (ResultVariable, ArrayLikeVariable)):
        # Special case: a ResultVariable which is the result will always be an array!
        annos["return"] = retvar = typing.List[retvar.type]
    varnames = list(props.varnames)
    defaults = list(props.defaults)
    diff = len(varnames) - len(defaults)

    safe_annos = annos.copy()
    retvars = [retvar] if retvar else []
    deletions = []
    for idx, name in enumerate(varnames):
        ann = safe_annos[name]
        if isinstance(ann, ArrayLikeVariable):
            ann = collections.abc.Sequence[ann.type]
            annos[name] = ann
        if not isinstance(ann, ResultVariable):
            continue
        # We move the variable to the end and remove it.
        # PYSIDE-1409: If the variable was the first arg, we move it to the front.
        # XXX This algorithm should probably be replaced by more introspection.
        retvars.insert(0 if idx == 0 else len(retvars), ann.type)
        deletions.append(idx)
        del annos[name]
    for idx in reversed(deletions):
        # varnames:  0 1 2 3 4 5 6 7
        # defaults:        0 1 2 3 4
        # diff: 3
        del varnames[idx]
        if idx >= diff:
            del defaults[idx - diff]
        else:
            diff -= 1
    if retvars:
        retvars = list(handle_retvar(rv) if isinstance(rv, ArrayLikeVariable) else rv
                       for rv in retvars)
        if len(retvars) == 1:
            returntype = retvars[0]
        else:
            retvars_str = ", ".join(map(to_string, retvars))
            typestr = f"typing.Tuple[{retvars_str}]"
            returntype = eval(typestr, globals(), namespace)
        props.annotations["return"] = returntype
    props.varnames = tuple(varnames)
    props.defaults = tuple(defaults)


def pyside_type_init(type_key, sig_strings):
    dprint()
    dprint(f"Initialization of type key '{type_key}'")
    update_mapping()
    ret = {}
    multi_props = []
    for line in sig_strings:
        props = calculate_props(line)
        shortname = props["name"]
        multi = props["multi"]
        if multi is None:
            ret[shortname] = props
            dprint(props)
        else:
            multi_props.append(props)
            if multi > 0:
                continue
            multi_props = {"multi": multi_props}
            ret[shortname] = multi_props
            dprint(multi_props)
            multi_props = []
    return ret

# end of file



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""Helper functions for formatting information on QMetaObject"""

from PySide6.QtCore import QMetaMethod


def _qbytearray_to_string(b):
    return bytes(b.data()).decode('utf-8')


def _format_metatype(meta_type):
    return meta_type.id() if meta_type.isValid() else '<invalid>'


def _dump_metaobject_helper(meta_obj, indent):
    meta_id = 0
    # FIXME: Otherwise crashes in Qt
    if meta_obj.propertyOffset() < meta_obj.propertyCount():
        meta_id = _format_metatype(meta_obj.metaType())
    print(f'{indent}class {meta_obj.className()}/{meta_id}:')
    indent += '    '

    info_offset = meta_obj.classInfoOffset()
    info_count = meta_obj.classInfoCount()
    if info_offset < info_count:
        print(f'{indent}Info:')
        for i in range(info_offset, info_count):
            name = meta_obj.classInfo(i).name()
            value = meta_obj.classInfo(i).value()
            print(f'{indent}{i:4d} {name}+{value}')

    enumerator_offset = meta_obj.enumeratorOffset()
    enumerator_count = meta_obj.enumeratorCount()
    if enumerator_offset < enumerator_count:
        print(f'{indent}Enumerators:')
        for e in range(enumerator_offset, enumerator_count):
            meta_enum = meta_obj.enumerator(e)
            name = meta_enum.name()
            value_str = ''
            descr = ''
            if meta_enum.isFlag():
                descr += ' flag'
            if meta_enum.isScoped():
                descr += ' scoped'
            for k in range(meta_enum.keyCount()):
                if k > 0:
                    value_str += ', '
                key = meta_enum.key(k)
                value = meta_enum.value(k)
                value_str += f'{key} = {value}'
            print(f'{indent}{e:4d} {name}{descr} ({value_str})')

    property_offset = meta_obj.propertyOffset()
    property_count = meta_obj.propertyCount()
    if property_offset < property_count:
        print(f'{indent}Properties:')
        for p in range(property_offset, property_count):
            meta_property = meta_obj.property(p)
            name = meta_property.name()
            desc = ''
            if meta_property.isConstant():
                desc += ', constant'
            if meta_property.isDesignable():
                desc += ', designable'
            if meta_property.isFlagType():
                desc += ', flag'
            if meta_property.isEnumType():
                desc += ', enum'
            if meta_property.isStored():
                desc += ', stored'
            if meta_property.isWritable():
                desc += ', writable'
            if meta_property.isResettable():
                desc += ', resettable'
            if meta_property.hasNotifySignal():
                notify_name_b = meta_property.notifySignal().name()
                notify_name = _qbytearray_to_string(notify_name_b)
                desc += f', notify="{notify_name}"'
            meta_id = _format_metatype(meta_property.metaType())
            type_name = meta_property.typeName()
            print(f'{indent}{p:4d} {type_name}/{meta_id} "{name}"{desc}')

    method_offset = meta_obj.methodOffset()
    method_count = meta_obj.methodCount()
    if method_offset < method_count:
        print('{}Methods:'.format(indent))
        for m in range(method_offset, method_count):
            method = meta_obj.method(m)
            signature = _qbytearray_to_string(method.methodSignature())
            access = ''
            if method.access() == QMetaMethod.Protected:
                access += 'protected '
            elif method.access() == QMetaMethod.Private:
                access += 'private '
            type = method.methodType()
            typeString = ''
            if type == QMetaMethod.Signal:
                typeString = ' (Signal)'
            elif type == QMetaMethod.Slot:
                typeString = ' (Slot)'
            elif type == QMetaMethod.Constructor:
                typeString = ' (Ct)'
            type_name = method.typeName()
            desc = f'{indent}{m:4d} {access}{type_name} "{signature}"{typeString}'
            parameter_names = method.parameterNames()
            if parameter_names:
                parameter_types = method.parameterTypes()
                desc += ' Parameters:'
                for p, bname in enumerate(parameter_names):
                    name = _qbytearray_to_string(bname)
                    type_name = _qbytearray_to_string(parameter_types[p])
                    if not name:
                        name = '<unnamed>'
                    desc += f' "{name}": {type_name}'
            print(desc)


def dump_metaobject(meta_obj):
    super_classes = [meta_obj]
    super_class = meta_obj.superClass()
    while super_class:
        super_classes.append(super_class)
        super_class = super_class.superClass()
    indent = ''
    for c in reversed(super_classes):
        _dump_metaobject_helper(c, indent)
        indent += '    '



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

# This file is automatically generated by Qt Design Studio.
import os
import sys
from pathlib import Path

from PySide6.QtQml import QQmlApplicationEngine

project_root = Path(__file__).parent.parent.parent


def setup_qt_environment(qml_engine: QQmlApplicationEngine):
    """
    Load the QML application. Import the compiled resources when the application is deployed.
    """
    qml_app_url = "DrumpadContent/App.qml"

    if "__compiled__" in globals():
        # Application has been deployed using pyside6-deploy
        try:
            import autogen.resources  # noqa: F401
        except ImportError:
            resource_file = Path(__file__).parent / "resources.py"
            print(
                f"Error: No compiled resources found in {resource_file.absolute()}\n"
                f"Please compile the resources using pyside6-rcc or pyside6-project build",
                file=sys.stderr,
            )
            sys.exit(1)

        qml_engine.addImportPath(":/")
        qml_engine.load(f":/{qml_app_url}")
        return

    qml_engine.addImportPath(str(project_root.absolute()))
    os.environ["QT_QUICK_CONTROLS_CONF"] = str(project_root / "qtquickcontrols2.conf")
    qml_engine.load(str(project_root / qml_app_url))



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
import urllib.request
import json

from PySide6.QtQuick import QQuickView
from PySide6.QtCore import QStringListModel
from PySide6.QtGui import QGuiApplication


if __name__ == '__main__':

    # get our data
    url = "http://country.io/names.json"
    response = urllib.request.urlopen(url)
    data = json.loads(response.read().decode('utf-8'))

    # Format and sort the data
    data_list = list(data.values())
    data_list.sort()

    # Set up the application window
    app = QGuiApplication(sys.argv)
    view = QQuickView()
    view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)

    # Expose the list to the Qml code
    my_model = QStringListModel()
    my_model.setStringList(data_list)
    view.setInitialProperties({"myModel": my_model})

    # Load the QML file
    # Add the current directory to the import paths and load the main module.
    view.engine().addImportPath(sys.path[0])
    view.loadFromModule("App", "Main")

    # Show the window
    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)
    view.show()

    # execute and cleanup
    app.exec()
    del view



#!/usr/bin/python
# Copyright (C) 2025 Ford Motor Company
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for dynamic source/replica types'''

import os
import sys
import unittest
from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtRemoteObjects import RepFile

from test_shared import wrap_tests_for_cleanup


contents = """
class Simple
{
    PROP(int i = 2);
    PROP(float f = -1. READWRITE);
    SIGNAL(random(int i));
    SLOT(void reset());
};
"""


@wrap_tests_for_cleanup(extra=['rep_file'])
class QDynamicReplicas(unittest.TestCase):
    '''Test case for dynamic Replicas'''

    def setUp(self):
        '''Set up test environment'''
        self.rep_file = RepFile(contents)

    def testDynamicReplica(self):
        '''Verify that a valid Replica is created'''
        Replica = self.rep_file.replica["Simple"]
        self.assertIsNotNone(Replica)
        replica = Replica()
        self.assertIsNotNone(replica)
        self.assertIsNotNone(replica.metaObject())
        meta = replica.metaObject()
        self.assertEqual(meta.className(), "Simple")
        self.assertEqual(meta.superClass().className(), "QRemoteObjectReplica")
        i = meta.indexOfProperty("i")
        self.assertNotEqual(i, -1)
        self.assertEqual(replica.propAsVariant(0), int(2))
        self.assertEqual(replica.propAsVariant(1), float(-1.0))
        self.assertEqual(replica.i, int(2))
        self.assertEqual(replica.f, float(-1.0))


@wrap_tests_for_cleanup(extra=['rep_file'])
class QDynamicSources(unittest.TestCase):
    '''Test case for dynamic Sources'''

    def setUp(self):
        '''Set up test environment'''
        self.rep_file = RepFile(contents)
        self.test_val = 0

    def on_changed(self, val):
        self.test_val = val

    def testDynamicSource(self):
        '''Verify that a valid Source is created'''
        Source = self.rep_file.source["Simple"]
        self.assertIsNotNone(Source)
        source = Source()
        self.assertIsNotNone(source)
        self.assertIsNotNone(source.metaObject())
        meta = source.metaObject()
        self.assertEqual(meta.className(), "SimpleSource")
        self.assertEqual(meta.superClass().className(), "QObject")
        i = meta.indexOfProperty("i")
        self.assertNotEqual(i, -1)
        self.assertIsNotNone(source.__dict__.get('__PROPERTIES__'))
        self.assertEqual(source.i, int(2))
        self.assertEqual(source.f, float(-1.0))
        source.iChanged.connect(self.on_changed)
        source.fChanged.connect(self.on_changed)
        source.i = 7
        self.assertEqual(source.i, int(7))
        self.assertEqual(self.test_val, int(7))
        source.i = 3
        self.assertEqual(self.test_val, int(3))
        source.f = 3.14
        self.assertAlmostEqual(self.test_val, float(3.14), places=5)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication, QHeaderView

received_column = None
received_order = None


def foo(colum, order):
    global received_column, received_order
    received_column = colum
    received_order = order


class TestBug941 (unittest.TestCase):

    def testIt(self):
        app = QApplication([])  # noqa: F841
        view = QHeaderView(Qt.Orientation.Horizontal)
        self.assertTrue(view.sortIndicatorChanged.connect(foo))
        # this can't raise an exception!
        view.sortIndicatorChanged.emit(0, Qt.SortOrder.DescendingOrder)
        self.assertEqual(received_column, 0)
        self.assertEqual(received_order, Qt.SortOrder.DescendingOrder)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QWidget, QHBoxLayout

from glwidget import GLWidget


class MainWindow(QWidget):
    def __init__(self):
        super().__init__()

        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        self._glwidget1 = GLWidget(self)
        layout.addWidget(self._glwidget1)
        self._glwidget2 = GLWidget(self)
        layout.addWidget(self._glwidget2)

    def closeEvent(self, event):
        self._glwidget1.stop_rendering()
        self._glwidget2.stop_rendering()
        event.accept()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QBuffer, Qt, QTimer  # noqa: E402
from PySide6.QtWidgets import QApplication, QWidget, QVBoxLayout  # noqa: E402
from PySide6.QtWebEngineWidgets import QWebEngineView  # noqa: E402
from PySide6.QtWebEngineCore import (QWebEngineProfile, QWebEngineUrlScheme,  # noqa: E402
                                     QWebEngineUrlSchemeHandler)  # noqa: E402


class TestSchemeHandler(QWebEngineUrlSchemeHandler):
    def requestStarted(self, request):
        if request.requestUrl() == "testpy:hello":
            request.redirect("testpy:goodbye")
            return

        self.buffer = QBuffer()
        self.buffer.setData(bytes("Really nice goodbye text.", "UTF-8"))
        self.buffer.aboutToClose.connect(self.buffer.deleteLater)
        request.reply(bytes("text/plain;charset=utf-8", "UTF-8"), self.buffer)


class MainTest(unittest.TestCase):
    def test_SchemeHandlerRedirect(self):
        self._loaded = False
        QApplication.setAttribute(Qt.ApplicationAttribute.AA_ShareOpenGLContexts)
        app = QApplication([])

        scheme_name = bytes("testpy", "UTF-8")
        scheme = QWebEngineUrlScheme(scheme_name)
        scheme.setSyntax(QWebEngineUrlScheme.Syntax.Path)
        QWebEngineUrlScheme.registerScheme(scheme)
        handler = TestSchemeHandler()
        profile = QWebEngineProfile.defaultProfile()
        profile.installUrlSchemeHandler(scheme_name, handler)

        top_level_widget = QWidget()
        top_level_widget.setWindowTitle('web_engine_custom_scheme.py')
        top_level_widget.resize(400, 400)
        layout = QVBoxLayout(top_level_widget)
        view = QWebEngineView()
        layout.addWidget(view)

        view.loadFinished.connect(self._slot_loaded)
        QTimer.singleShot(5000, app.quit)

        top_level_widget.show()
        view.load("testpy:hello")
        app.exec()

        self.assertTrue(self._loaded)
        self.assertEqual(view.url(), "testpy:goodbye")

    def _slot_loaded(self):
        self._loaded = True
        QApplication.quit()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test the QShortcut constructor'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QGuiApplication, QKeySequence, QShortcut, QWindow


class Foo(QWindow):
    def __init__(self):
        super().__init__()
        self.ok = False
        self.copy = False

    def slot_of_foo(self):
        self.ok = True

    def slot_of_copy(self):
        self.copy = True


class MyShortcut(QShortcut):
    def __init__(self, keys, wdg, slot):
        QShortcut.__init__(self, keys, wdg, slot)

    def emit_signal(self):
        self.activated.emit()


class QAppPresence(unittest.TestCase):

    def testQShortcut(self):
        self.qapp = QGuiApplication([])
        f = Foo()

        self.sc = MyShortcut(QKeySequence(Qt.Key.Key_Return), f, f.slot_of_foo)
        self.scstd = MyShortcut(QKeySequence.StandardKey.Copy, f, f.slot_of_copy)
        QTimer.singleShot(0, self.init)
        self.qapp.exec()
        self.assertEqual(f.ok, True)
        self.assertEqual(f.copy, True)

    def init(self):
        self.sc.emit_signal()
        self.scstd.emit_signal()
        self.qapp.quit()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the qml/examples/qml/referenceexamples/adding example from Qt v6.x"""

from pathlib import Path
import sys

from PySide6.QtCore import QCoreApplication
from PySide6.QtQml import QQmlComponent, QQmlEngine

from person import Person  # noqa: F401


if __name__ == '__main__':
    app = QCoreApplication(sys.argv)

    engine = QQmlEngine()
    engine.addImportPath(Path(__file__).parent)
    component = QQmlComponent(engine)
    component.loadFromModule("People", "Main")

    if person := component.create():
        print(f"The person's name is {person.name}")
        print(f"They wear a {person.shoe_size} sized shoe")
    else:
        print(component.errors())
    del engine
    sys.exit(0)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QPushButton, QSpinBox

from helper.basicpyslotcase import BasicPySlotCase
from helper.usesqapplication import UsesQApplication


class QtGuiMultipleSlots(UsesQApplication):
    '''Multiple connections to QtGui signals'''

    def run_many(self, signal, emitter, receivers, args=None):
        """Utility method to connect a list of receivers to a signal.
        sender - QObject that will emit the signal
        signal - string with the signal signature
        emitter - the callable that will trigger the signal
        receivers - list of BasicPySlotCase instances
        args - tuple with the arguments to be sent.
        """

        if args is None:
            args = tuple()

        for rec in receivers:
            rec.setUp()
            signal.connect(rec.cb)
            rec.args = tuple(args)

        emitter(*args)

        for rec in receivers:
            self.assertTrue(rec.called)

    def testButtonClick(self):
        """Multiple connections to QPushButton.clicked()"""
        sender = QPushButton('button')
        receivers = [BasicPySlotCase() for x in range(30)]
        self.run_many(sender.clicked, sender.click, receivers)

    def testSpinBoxValueChanged(self):
        """Multiple connections to QSpinBox.valueChanged(int)"""
        sender = QSpinBox()
        # FIXME if number of receivers if higher than 50, segfaults
        receivers = [BasicPySlotCase() for x in range(10)]
        self.run_many(sender.valueChanged, sender.setValue,
                      receivers, (1,))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'downloadmanagerwidget.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QLabel, QLayout, QScrollArea,
    QSizePolicy, QVBoxLayout, QWidget)

class Ui_DownloadManagerWidget(object):
    def setupUi(self, DownloadManagerWidget):
        if not DownloadManagerWidget.objectName():
            DownloadManagerWidget.setObjectName(u"DownloadManagerWidget")
        DownloadManagerWidget.resize(400, 212)
        DownloadManagerWidget.setStyleSheet(u"#DownloadManagerWidget {\n"
"    background: palette(button)\n"
"}")
        self.m_topLevelLayout = QVBoxLayout(DownloadManagerWidget)
        self.m_topLevelLayout.setObjectName(u"m_topLevelLayout")
        self.m_topLevelLayout.setSizeConstraint(QLayout.SizeConstraint.SetNoConstraint)
        self.m_topLevelLayout.setContentsMargins(0, 0, 0, 0)
        self.m_scrollArea = QScrollArea(DownloadManagerWidget)
        self.m_scrollArea.setObjectName(u"m_scrollArea")
        self.m_scrollArea.setStyleSheet(u"#m_scrollArea {\n"
"  margin: 2px;\n"
"  border: none;\n"
"}")
        self.m_scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.m_scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.m_scrollArea.setWidgetResizable(True)
        self.m_scrollArea.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.m_items = QWidget()
        self.m_items.setObjectName(u"m_items")
        self.m_items.setGeometry(QRect(0, 0, 382, 208))
        self.m_items.setStyleSheet(u"#m_items {background: palette(mid)}")
        self.m_itemsLayout = QVBoxLayout(self.m_items)
        self.m_itemsLayout.setSpacing(2)
        self.m_itemsLayout.setObjectName(u"m_itemsLayout")
        self.m_itemsLayout.setContentsMargins(3, 3, 3, 3)
        self.m_zeroItemsLabel = QLabel(self.m_items)
        self.m_zeroItemsLabel.setObjectName(u"m_zeroItemsLabel")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.m_zeroItemsLabel.sizePolicy().hasHeightForWidth())
        self.m_zeroItemsLabel.setSizePolicy(sizePolicy)
        self.m_zeroItemsLabel.setStyleSheet(u"color: palette(shadow)")
        self.m_zeroItemsLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.m_itemsLayout.addWidget(self.m_zeroItemsLabel)

        self.m_scrollArea.setWidget(self.m_items)

        self.m_topLevelLayout.addWidget(self.m_scrollArea)


        self.retranslateUi(DownloadManagerWidget)

        QMetaObject.connectSlotsByName(DownloadManagerWidget)
    # setupUi

    def retranslateUi(self, DownloadManagerWidget):
        DownloadManagerWidget.setWindowTitle(QCoreApplication.translate("DownloadManagerWidget", u"Downloads", None))
        self.m_zeroItemsLabel.setText(QCoreApplication.translate("DownloadManagerWidget", u"No downloads", None))
    # retranslateUi




# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
import weakref
import math

from PySide6.QtCore import (QLineF, QPointF, QRandomGenerator, QRectF, QSizeF,
                            Qt, qAbs)
from PySide6.QtGui import (QColor, QBrush, QLinearGradient, QPainter, QPainterPath, QPen,
                           QPolygonF, QRadialGradient)
from PySide6.QtWidgets import (QApplication, QGraphicsItem, QGraphicsScene,
                               QGraphicsView, QStyle)


def random(boundary):
    return QRandomGenerator.global_().bounded(boundary)


class Edge(QGraphicsItem):

    def __init__(self, sourceNode, destNode):
        super().__init__()

        self._arrow_size = 10.0
        self._source_point = QPointF()
        self._dest_point = QPointF()
        self.setAcceptedMouseButtons(Qt.MouseButton.NoButton)
        self.source = weakref.ref(sourceNode)
        self.dest = weakref.ref(destNode)
        self.source().add_edge(self)
        self.dest().add_edge(self)
        self.adjust()

    def item_type(self):
        return QGraphicsItem.UserType + 2

    def source_node(self):
        return self.source()

    def set_source_node(self, node):
        self.source = weakref.ref(node)
        self.adjust()

    def dest_node(self):
        return self.dest()

    def set_dest_node(self, node):
        self.dest = weakref.ref(node)
        self.adjust()

    def adjust(self):
        if not self.source() or not self.dest():
            return

        line = QLineF(self.mapFromItem(self.source(), 0, 0),
                      self.mapFromItem(self.dest(), 0, 0))
        length = line.length()

        if length == 0.0:
            return

        edge_offset = QPointF((line.dx() * 10) / length, (line.dy() * 10) / length)

        self.prepareGeometryChange()
        self._source_point = line.p1() + edge_offset
        self._dest_point = line.p2() - edge_offset

    def boundingRect(self):
        if not self.source() or not self.dest():
            return QRectF()

        pen_width = 1
        extra = (pen_width + self._arrow_size) / 2.0

        width = self._dest_point.x() - self._source_point.x()
        height = self._dest_point.y() - self._source_point.y()
        rect = QRectF(self._source_point, QSizeF(width, height))
        return rect.normalized().adjusted(-extra, -extra, extra, extra)

    def paint(self, painter, option, widget):
        if not self.source() or not self.dest():
            return

        # Draw the line itself.
        line = QLineF(self._source_point, self._dest_point)

        if line.length() == 0.0:
            return

        painter.setPen(QPen(Qt.GlobalColor.black, 1, Qt.PenStyle.SolidLine,
                            Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin))
        painter.drawLine(line)

        # Draw the arrows if there's enough room.
        angle = math.acos(line.dx() / line.length())
        if line.dy() >= 0:
            angle = 2 * math.pi - angle

        arrow_head1 = QPointF(math.sin(angle + math.pi / 3) * self._arrow_size,
                              math.cos(angle + math.pi / 3) * self._arrow_size)
        source_arrow_p1 = self._source_point + arrow_head1
        arrow_head2 = QPointF(math.sin(angle + math.pi - math.pi / 3) * self._arrow_size,
                              math.cos(angle + math.pi - math.pi / 3) * self._arrow_size)
        source_arrow_p2 = self._source_point + arrow_head2

        arrow_head1 = QPointF(math.sin(angle - math.pi / 3) * self._arrow_size,
                              math.cos(angle - math.pi / 3) * self._arrow_size)
        dest_arrow_p1 = self._dest_point + arrow_head1
        arrow_head2 = QPointF(math.sin(angle - math.pi + math.pi / 3) * self._arrow_size,
                              math.cos(angle - math.pi + math.pi / 3) * self._arrow_size)
        dest_arrow_p2 = self._dest_point + arrow_head2

        painter.setBrush(Qt.GlobalColor.black)
        painter.drawPolygon(QPolygonF([line.p1(), source_arrow_p1, source_arrow_p2]))
        painter.drawPolygon(QPolygonF([line.p2(), dest_arrow_p1, dest_arrow_p2]))


class Node(QGraphicsItem):

    def __init__(self, graphWidget):
        super().__init__()

        self.graph = weakref.ref(graphWidget)
        self._edge_list = []
        self._new_pos = QPointF()
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable)
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges)
        self.setCacheMode(QGraphicsItem.CacheMode.DeviceCoordinateCache)
        self.setZValue(-1)

    def item_type(self):
        return QGraphicsItem.UserType + 1

    def add_edge(self, edge):
        self._edge_list.append(weakref.ref(edge))
        edge.adjust()

    def edges(self):
        return self._edge_list

    def calculate_forces(self):
        if not self.scene() or self.scene().mouseGrabberItem() is self:
            self._new_pos = self.pos()
            return

        # Sum up all forces pushing this item away.
        xvel = 0.0
        yvel = 0.0
        for item in self.scene().items():
            if not isinstance(item, Node):
                continue

            line = QLineF(self.mapFromItem(item, 0, 0), QPointF(0, 0))
            dx = line.dx()
            dy = line.dy()
            l = 2.0 * (dx * dx + dy * dy)  # noqa: E741
            if l > 0:
                xvel += (dx * 150.0) / l
                yvel += (dy * 150.0) / l

        # Now subtract all forces pulling items together.
        weight = (len(self._edge_list) + 1) * 10.0
        for edge in self._edge_list:
            if edge().source_node() is self:
                pos = self.mapFromItem(edge().dest_node(), 0, 0)
            else:
                pos = self.mapFromItem(edge().source_node(), 0, 0)
            xvel += pos.x() / weight
            yvel += pos.y() / weight

        if qAbs(xvel) < 0.1 and qAbs(yvel) < 0.1:
            xvel = yvel = 0.0

        scene_rect = self.scene().sceneRect()
        self._new_pos = self.pos() + QPointF(xvel, yvel)
        self._new_pos.setX(min(max(self._new_pos.x(), scene_rect.left() + 10),
                               scene_rect.right() - 10))
        self._new_pos.setY(min(max(self._new_pos.y(), scene_rect.top() + 10),
                               scene_rect.bottom() - 10))

    def advance(self):
        if self._new_pos == self.pos():
            return False

        self.setPos(self._new_pos)
        return True

    def boundingRect(self):
        adjust = 2.0
        return QRectF(-10 - adjust, -10 - adjust,
                      23 + adjust, 23 + adjust)

    def shape(self):
        path = QPainterPath()
        path.addEllipse(-10, -10, 20, 20)
        return path

    def paint(self, painter, option, widget):
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(Qt.GlobalColor.darkGray)
        painter.drawEllipse(-7, -7, 20, 20)

        gradient = QRadialGradient(-3, -3, 10)
        if option.state & QStyle.StateFlag.State_Sunken:
            gradient.setCenter(3, 3)
            gradient.setFocalPoint(3, 3)
            gradient.setColorAt(1, QColor(Qt.GlobalColor.yellow).lighter(120))
            gradient.setColorAt(0, QColor(Qt.GlobalColor.darkYellow).lighter(120))
        else:
            gradient.setColorAt(0, Qt.GlobalColor.yellow)
            gradient.setColorAt(1, Qt.GlobalColor.darkYellow)

        painter.setBrush(QBrush(gradient))
        painter.setPen(QPen(Qt.GlobalColor.black, 0))
        painter.drawEllipse(-10, -10, 20, 20)

    def itemChange(self, change, value):
        if change == QGraphicsItem.GraphicsItemChange.ItemPositionChange:
            for edge in self._edge_list:
                edge().adjust()
            self.graph().item_moved()

        return QGraphicsItem.itemChange(self, change, value)

    def mousePressEvent(self, event):
        self.update()
        QGraphicsItem.mousePressEvent(self, event)

    def mouseReleaseEvent(self, event):
        self.update()
        QGraphicsItem.mouseReleaseEvent(self, event)


class GraphWidget(QGraphicsView):
    def __init__(self):
        super().__init__()

        self._timer_id = 0

        scene = QGraphicsScene(self)
        scene.setItemIndexMethod(QGraphicsScene.ItemIndexMethod.NoIndex)
        scene.setSceneRect(-200, -200, 400, 400)
        self.setScene(scene)
        self.setCacheMode(QGraphicsView.CacheModeFlag.CacheBackground)
        self.setRenderHint(QPainter.RenderHint.Antialiasing)
        self.setTransformationAnchor(QGraphicsView.ViewportAnchor.AnchorUnderMouse)
        self.setResizeAnchor(QGraphicsView.ViewportAnchor.AnchorViewCenter)

        node1 = Node(self)
        node2 = Node(self)
        node3 = Node(self)
        node4 = Node(self)
        self._center_node = Node(self)
        node6 = Node(self)
        node7 = Node(self)
        node8 = Node(self)
        node9 = Node(self)
        scene.addItem(node1)
        scene.addItem(node2)
        scene.addItem(node3)
        scene.addItem(node4)
        scene.addItem(self._center_node)
        scene.addItem(node6)
        scene.addItem(node7)
        scene.addItem(node8)
        scene.addItem(node9)
        scene.addItem(Edge(node1, node2))
        scene.addItem(Edge(node2, node3))
        scene.addItem(Edge(node2, self._center_node))
        scene.addItem(Edge(node3, node6))
        scene.addItem(Edge(node4, node1))
        scene.addItem(Edge(node4, self._center_node))
        scene.addItem(Edge(self._center_node, node6))
        scene.addItem(Edge(self._center_node, node8))
        scene.addItem(Edge(node6, node9))
        scene.addItem(Edge(node7, node4))
        scene.addItem(Edge(node8, node7))
        scene.addItem(Edge(node9, node8))

        node1.setPos(-50, -50)
        node2.setPos(0, -50)
        node3.setPos(50, -50)
        node4.setPos(-50, 0)
        self._center_node.setPos(0, 0)
        node6.setPos(50, 0)
        node7.setPos(-50, 50)
        node8.setPos(0, 50)
        node9.setPos(50, 50)

        self.scale(0.8, 0.8)
        self.setMinimumSize(400, 400)
        self.setWindowTitle(self.tr("Elastic Nodes"))

    def item_moved(self):
        if not self._timer_id:
            self._timer_id = self.startTimer(1000 / 25)

    def keyPressEvent(self, event):
        key = event.key()

        if key == Qt.Key.Key_Up:
            self._center_node.moveBy(0, -20)
        elif key == Qt.Key.Key_Down:
            self._center_node.moveBy(0, 20)
        elif key == Qt.Key.Key_Left:
            self._center_node.moveBy(-20, 0)
        elif key == Qt.Key.Key_Right:
            self._center_node.moveBy(20, 0)
        elif key == Qt.Key.Key_Plus:
            self.scale_view(1.2)
        elif key == Qt.Key.Key_Minus:
            self.scale_view(1 / 1.2)
        elif key == Qt.Key.Key_Space or key == Qt.Key.Key_Enter:
            for item in self.scene().items():
                if isinstance(item, Node):
                    item.setPos(-150 + random(300), -150 + random(300))
        else:
            QGraphicsView.keyPressEvent(self, event)

    def timerEvent(self, event):
        nodes = [item for item in self.scene().items() if isinstance(item, Node)]

        for node in nodes:
            node.calculate_forces()

        items_moved = False
        for node in nodes:
            if node.advance():
                items_moved = True

        if not items_moved:
            self.killTimer(self._timer_id)
            self._timer_id = 0

    def wheelEvent(self, event):
        delta = event.angleDelta().y()
        self.scale_view(math.pow(2.0, -delta / 240.0))

    def draw_background(self, painter, rect):
        # Shadow.
        scene_rect = self.sceneRect()
        right_shadow = QRectF(scene_rect.right(), scene_rect.top() + 5,
                              5, scene_rect.height())
        bottom_shadow = QRectF(scene_rect.left() + 5, scene_rect.bottom(),
                               scene_rect.width(), 5)
        if right_shadow.intersects(rect) or right_shadow.contains(rect):
            painter.fillRect(right_shadow, Qt.darkGray)
        if bottom_shadow.intersects(rect) or bottom_shadow.contains(rect):
            painter.fillRect(bottom_shadow, Qt.darkGray)

        # Fill.
        gradient = QLinearGradient(scene_rect.topLeft(), scene_rect.bottomRight())
        gradient.setColorAt(0, Qt.GlobalColor.white)
        gradient.setColorAt(1, Qt.lightGray)
        painter.fillRect(rect.intersected(scene_rect), QBrush(gradient))
        painter.setBrush(Qt.NoBrush)
        painter.drawRect(scene_rect)

        # Text.
        text_rect = QRectF(scene_rect.left() + 4, scene_rect.top() + 4,
                           scene_rect.width() - 4, scene_rect.height() - 4)
        message = self.tr("Click and drag the nodes around, and zoom with the "
                          "mouse wheel or the '+' and '-' keys")

        font = painter.font()
        font.setBold(True)
        font.setPointSize(14)
        painter.setFont(font)
        painter.setPen(Qt.lightGray)
        painter.drawText(text_rect.translated(2, 2), message)
        painter.setPen(Qt.GlobalColor.black)
        painter.drawText(text_rect, message)

    def scale_view(self, scaleFactor):
        factor = self.transform().scale(scaleFactor, scaleFactor).mapRect(
            QRectF(0, 0, 1, 1)).width()

        if factor < 0.07 or factor > 100:
            return

        self.scale(scaleFactor, scaleFactor)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    widget = GraphWidget()
    widget.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QLayout handling of child widgets references'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QGridLayout, QWidget
from PySide6.QtWidgets import QStackedLayout, QFormLayout
from PySide6.QtWidgets import QPushButton, QLabel

from helper.usesqapplication import UsesQApplication


class SaveReference(UsesQApplication):
    '''Test case to check if QLayout-derived classes increment the refcount
    of widgets passed to addWidget()'''

    # Adding here as nose can't see the qapplication attrib we inherit
    qapplication = True

    def setUp(self):
        # Acquire resources
        super(SaveReference, self).setUp()
        self.widget1 = QPushButton('click me')
        self.widget2 = QLabel('aaa')

    def tearDown(self):
        # Release resources
        del self.widget2
        del self.widget1
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(SaveReference, self).tearDown()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def checkLayoutReference(self, layout):
        # Checks the reference cound handling of layout.addWidget
        self.assertEqual(sys.getrefcount(self.widget1), 2)
        layout.addWidget(self.widget1)
        self.assertEqual(sys.getrefcount(self.widget1), 3)

        self.assertEqual(sys.getrefcount(self.widget2), 2)
        layout.addWidget(self.widget2)
        self.assertEqual(sys.getrefcount(self.widget2), 3)

        # Check if doesn't mess around with previous widget refcount
        self.assertEqual(sys.getrefcount(self.widget1), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testMoveLayout(self):
        layout = QHBoxLayout()
        self.assertEqual(sys.getrefcount(self.widget1), 2)
        layout.addWidget(self.widget1)
        self.assertEqual(sys.getrefcount(self.widget1), 3)

        w = QWidget()
        w.setLayout(layout)
        self.assertEqual(sys.getrefcount(self.widget1), 3)

    def testHBoxReference(self):
        # QHBoxLayout.addWidget reference count
        w = QWidget()
        self.checkLayoutReference(QHBoxLayout(w))

    def testVBoxReference(self):
        # QVBoxLayout.addWidget reference count
        w = QWidget()
        self.checkLayoutReference(QVBoxLayout(w))

    def testGridReference(self):
        # QGridLayout.addWidget reference count
        w = QWidget()
        self.checkLayoutReference(QGridLayout(w))

    def testFormReference(self):
        # QFormLayout.addWidget reference count
        w = QWidget()
        self.checkLayoutReference(QFormLayout(w))

    def testStackedReference(self):
        # QStackedLayout.addWidget reference count
        w = QWidget()
        self.checkLayoutReference(QStackedLayout(w))


class MultipleAdd(UsesQApplication):
    '''Test case to check if refcount is incremented only once when multiple
    calls to addWidget are made with the same widget'''

    qapplication = True

    def setUp(self):
        # Acquire resources
        super(MultipleAdd, self).setUp()
        self.widget = QPushButton('click me')
        self.win = QWidget()
        self.layout = QHBoxLayout(self.win)

    def tearDown(self):
        # Release resources
        del self.widget
        del self.layout
        del self.win
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(MultipleAdd, self).tearDown()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCount(self):
        # Multiple QLayout.addWidget calls on the same widget
        self.assertEqual(sys.getrefcount(self.widget), 2)
        self.layout.addWidget(self.widget)
        self.assertEqual(sys.getrefcount(self.widget), 3)
        self.layout.addWidget(self.widget)
        self.assertEqual(sys.getrefcount(self.widget), 3)
        self.layout.addWidget(self.widget)
        self.assertEqual(sys.getrefcount(self.widget), 3)


class InternalAdd(UsesQApplication):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testInternalRef(self):
        mw = QWidget()
        w = QWidget()
        ow = QWidget()

        topLayout = QGridLayout()

        # unique reference
        base_ref_count_w = sys.getrefcount(w)
        base_ref_count_ow = sys.getrefcount(ow)

        topLayout.addWidget(w, 0, 0)
        topLayout.addWidget(ow, 1, 0)

        # layout keep the referemce
        self.assertEqual(sys.getrefcount(w), base_ref_count_w + 1)
        self.assertEqual(sys.getrefcount(ow), base_ref_count_ow + 1)

        mainLayout = QGridLayout()

        mainLayout.addLayout(topLayout, 1, 0, 1, 4)

        # the same reference
        self.assertEqual(sys.getrefcount(w), base_ref_count_w + 1)
        self.assertEqual(sys.getrefcount(ow), base_ref_count_ow + 1)

        mw.setLayout(mainLayout)

        # now trasfer the ownership to mw
        self.assertEqual(sys.getrefcount(w), base_ref_count_w + 1)
        self.assertEqual(sys.getrefcount(ow), base_ref_count_ow + 1)

        del mw

        # remove the ref and invalidate the widget
        self.assertEqual(sys.getrefcount(w), base_ref_count_w)
        self.assertEqual(sys.getrefcount(ow), base_ref_count_ow)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from minimal import MinBoolUser


class DerivedMinBoolUser (MinBoolUser):
    def returnMyselfVirtual(self):
        return MinBoolUser()


class MinBoolTest(unittest.TestCase):

    def testMinBoolUser(self):
        mbuTrue = MinBoolUser()
        mbuFalse = MinBoolUser()
        mbuTrue.setMinBool(True)
        self.assertFalse(mbuFalse.minBool())
        self.assertTrue(mbuTrue.minBool())
        self.assertFalse(mbuTrue.callInvertedMinBool())

        self.assertTrue(mbuTrue.minBool())
        self.assertFalse(mbuFalse.minBool())
        self.assertTrue(mbuTrue.minBool() != mbuFalse.minBool())

        self.assertFalse(mbuFalse.minBool())
        self.assertFalse(mbuFalse.minBool())
        self.assertTrue(mbuTrue.minBool() != mbuFalse.minBool())

    def testVirtuals(self):
        dmbu = DerivedMinBoolUser()
        self.assertEqual(dmbu.invertedMinBool(), True)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QWidget, QMenu
from helper.usesqapplication import UsesQApplication


class QPainterDrawText(UsesQApplication):

    def _cb(self):
        self._called = True

    def testSignal(self):
        o = QWidget()
        act = QAction(o)
        self._called = False
        act.triggered.connect(self._cb)
        act.trigger()
        self.assertTrue(self._called)

    def testNewCtor(self):
        o = QWidget()
        self._called = False
        myAction = QAction("&Quit", o, triggered=self._cb)
        myAction.trigger()
        self.assertTrue(self._called)


class SetShortcutTest(UsesQApplication):

    def testSetShortcut(self):
        # Somehow an exception was leaking from the constructor
        # and appearing in setShortcut.
        o = QWidget()
        action = QAction('aaaa', o)
        shortcut = 'Ctrl+N'
        action.setShortcut(shortcut)
        s2 = action.shortcut()
        self.assertEqual(s2, shortcut)

    def testMenu(self):
        # Test the setMenu()/menu() old functionality removed in Qt6
        # that was added via helper functions.
        menu = QMenu("menu")
        action = QAction("action")

        # Using QAction::setMenu(QObject*)
        action.setMenu(menu)

        self.assertEqual(action.menu(), menu)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""Test cases for QtAsyncio"""

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import asyncio
import PySide6.QtAsyncio as QtAsyncio


@unittest.skipIf(sys.version_info < (3, 11), "Requires ExceptionGroup")
class QAsyncioTestCaseBug2799(unittest.TestCase):
    async def job(self):
        await asyncio.sleep(1)

    async def main(self):
        async with asyncio.TaskGroup() as tg:
            tg.create_task(self.job())
            raise RuntimeError()

    def test_exception_group(self):
        with self.assertRaises(ExceptionGroup):  # noqa: F821
            QtAsyncio.run(self.main(), keep_running=False)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'bookwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QComboBox, QFormLayout,
    QGroupBox, QHeaderView, QLabel, QLineEdit,
    QMainWindow, QSizePolicy, QSpinBox, QTableView,
    QVBoxLayout, QWidget)

class Ui_BookWindow(object):
    def setupUi(self, BookWindow):
        if not BookWindow.objectName():
            BookWindow.setObjectName(u"BookWindow")
        BookWindow.resize(601, 420)
        self.centralWidget = QWidget(BookWindow)
        self.centralWidget.setObjectName(u"centralWidget")
        self.vboxLayout = QVBoxLayout(self.centralWidget)
        self.vboxLayout.setSpacing(6)
        self.vboxLayout.setObjectName(u"vboxLayout")
        self.vboxLayout.setContentsMargins(9, 9, 9, 9)
        self.groupBox = QGroupBox(self.centralWidget)
        self.groupBox.setObjectName(u"groupBox")
        self.vboxLayout1 = QVBoxLayout(self.groupBox)
        self.vboxLayout1.setSpacing(6)
        self.vboxLayout1.setObjectName(u"vboxLayout1")
        self.vboxLayout1.setContentsMargins(9, 9, 9, 9)
        self.bookTable = QTableView(self.groupBox)
        self.bookTable.setObjectName(u"bookTable")
        self.bookTable.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)

        self.vboxLayout1.addWidget(self.bookTable)

        self.groupBox_2 = QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.formLayout = QFormLayout(self.groupBox_2)
        self.formLayout.setObjectName(u"formLayout")
        self.label_5 = QLabel(self.groupBox_2)
        self.label_5.setObjectName(u"label_5")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label_5)

        self.titleEdit = QLineEdit(self.groupBox_2)
        self.titleEdit.setObjectName(u"titleEdit")
        self.titleEdit.setEnabled(True)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.titleEdit)

        self.label_2 = QLabel(self.groupBox_2)
        self.label_2.setObjectName(u"label_2")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_2)

        self.authorEdit = QComboBox(self.groupBox_2)
        self.authorEdit.setObjectName(u"authorEdit")
        self.authorEdit.setEnabled(True)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.authorEdit)

        self.label_3 = QLabel(self.groupBox_2)
        self.label_3.setObjectName(u"label_3")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.label_3)

        self.genreEdit = QComboBox(self.groupBox_2)
        self.genreEdit.setObjectName(u"genreEdit")
        self.genreEdit.setEnabled(True)

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.genreEdit)

        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.label_4)

        self.yearEdit = QSpinBox(self.groupBox_2)
        self.yearEdit.setObjectName(u"yearEdit")
        self.yearEdit.setEnabled(True)
        self.yearEdit.setMinimum(-1000)
        self.yearEdit.setMaximum(2100)

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.yearEdit)

        self.label = QLabel(self.groupBox_2)
        self.label.setObjectName(u"label")

        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.label)

        self.ratingEdit = QSpinBox(self.groupBox_2)
        self.ratingEdit.setObjectName(u"ratingEdit")
        self.ratingEdit.setMaximum(5)

        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.ratingEdit)


        self.vboxLayout1.addWidget(self.groupBox_2)


        self.vboxLayout.addWidget(self.groupBox)

        BookWindow.setCentralWidget(self.centralWidget)
        QWidget.setTabOrder(self.bookTable, self.titleEdit)
        QWidget.setTabOrder(self.titleEdit, self.authorEdit)
        QWidget.setTabOrder(self.authorEdit, self.genreEdit)
        QWidget.setTabOrder(self.genreEdit, self.yearEdit)

        self.retranslateUi(BookWindow)

        QMetaObject.connectSlotsByName(BookWindow)
    # setupUi

    def retranslateUi(self, BookWindow):
        BookWindow.setWindowTitle(QCoreApplication.translate("BookWindow", u"Books", None))
        self.groupBox.setTitle("")
        self.groupBox_2.setTitle(QCoreApplication.translate("BookWindow", u"Details", None))
        self.label_5.setText(QCoreApplication.translate("BookWindow", u"<b>Title:</b>", None))
        self.label_2.setText(QCoreApplication.translate("BookWindow", u"<b>Author: </b>", None))
        self.label_3.setText(QCoreApplication.translate("BookWindow", u"<b>Genre:</b>", None))
        self.label_4.setText(QCoreApplication.translate("BookWindow", u"<b>Year:</b>", None))
        self.yearEdit.setPrefix("")
        self.label.setText(QCoreApplication.translate("BookWindow", u"<b>Rating:</b>", None))
    # retranslateUi




#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations
import sys
import os
import subprocess
from pathlib import Path


def main():
    # The tools listed as entrypoints in setup.py are copied to 'scripts/..'
    cmd = Path("..") / Path(sys.argv[0]).name
    command = [os.fspath(Path(__file__).parent.resolve() / cmd)]
    command.extend(sys.argv[1:])
    sys.exit(subprocess.call(command))


def genpyi():
    # After we changed the shibokensupport module to be totally virtual,
    # it is no longer possible to call the pyi generator from the file system.
    command = [sys.executable, "-c",
               "import shiboken6;"
               "from shibokensupport.signature.lib.pyi_generator import main;"
               "main()"] + sys.argv[1:]
    sys.exit(subprocess.call(command))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtSerialBus'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QFile  # noqa: E402
from PySide6.QtSerialBus import QCanDbcFileParser  # noqa: E402


class QSerialBusTest(unittest.TestCase):
    def setUp(self):
        self.dbc_file = Path(__file__).parent / "test.dbc"

    def test_qfile_open(self):
        f = QFile(str(self.dbc_file))
        self.assertTrue(f.open(QFile.OpenModeFlag.ReadOnly), msg=f.errorString())
        f.close()

    def test_qcandbcfileparser_parse(self):
        parser = QCanDbcFileParser()

        self.assertTrue(parser.parse([str(self.dbc_file)]), msg=parser.errorString())

        self.assertTrue(parser.parse(str(self.dbc_file)), msg=parser.errorString())

        self.assertTrue(parser.parse([str(self.dbc_file.resolve())]), msg=parser.errorString())
        self.assertTrue(parser.parse(str(self.dbc_file.resolve())), msg=parser.errorString())

        self.assertFalse(parser.parse(["."]), msg=parser.errorString())


if __name__ == '__main__':
    unittest.main()
